# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.terminal._configuration_create_params import (
        ConfigurationCreateParams as ConfigurationCreateParams,
        ConfigurationCreateParamsBbposWisepad3 as ConfigurationCreateParamsBbposWisepad3,
        ConfigurationCreateParamsBbposWiseposE as ConfigurationCreateParamsBbposWiseposE,
        ConfigurationCreateParamsOffline as ConfigurationCreateParamsOffline,
        ConfigurationCreateParamsRebootWindow as ConfigurationCreateParamsRebootWindow,
        ConfigurationCreateParamsStripeS700 as ConfigurationCreateParamsStripeS700,
        ConfigurationCreateParamsTipping as ConfigurationCreateParamsTipping,
        ConfigurationCreateParamsTippingAed as ConfigurationCreateParamsTippingAed,
        ConfigurationCreateParamsTippingAud as ConfigurationCreateParamsTippingAud,
        ConfigurationCreateParamsTippingBgn as ConfigurationCreateParamsTippingBgn,
        ConfigurationCreateParamsTippingCad as ConfigurationCreateParamsTippingCad,
        ConfigurationCreateParamsTippingChf as ConfigurationCreateParamsTippingChf,
        ConfigurationCreateParamsTippingCzk as ConfigurationCreateParamsTippingCzk,
        ConfigurationCreateParamsTippingDkk as ConfigurationCreateParamsTippingDkk,
        ConfigurationCreateParamsTippingEur as ConfigurationCreateParamsTippingEur,
        ConfigurationCreateParamsTippingGbp as ConfigurationCreateParamsTippingGbp,
        ConfigurationCreateParamsTippingGip as ConfigurationCreateParamsTippingGip,
        ConfigurationCreateParamsTippingHkd as ConfigurationCreateParamsTippingHkd,
        ConfigurationCreateParamsTippingHuf as ConfigurationCreateParamsTippingHuf,
        ConfigurationCreateParamsTippingJpy as ConfigurationCreateParamsTippingJpy,
        ConfigurationCreateParamsTippingMxn as ConfigurationCreateParamsTippingMxn,
        ConfigurationCreateParamsTippingMyr as ConfigurationCreateParamsTippingMyr,
        ConfigurationCreateParamsTippingNok as ConfigurationCreateParamsTippingNok,
        ConfigurationCreateParamsTippingNzd as ConfigurationCreateParamsTippingNzd,
        ConfigurationCreateParamsTippingPln as ConfigurationCreateParamsTippingPln,
        ConfigurationCreateParamsTippingRon as ConfigurationCreateParamsTippingRon,
        ConfigurationCreateParamsTippingSek as ConfigurationCreateParamsTippingSek,
        ConfigurationCreateParamsTippingSgd as ConfigurationCreateParamsTippingSgd,
        ConfigurationCreateParamsTippingUsd as ConfigurationCreateParamsTippingUsd,
        ConfigurationCreateParamsVerifoneP400 as ConfigurationCreateParamsVerifoneP400,
        ConfigurationCreateParamsWifi as ConfigurationCreateParamsWifi,
        ConfigurationCreateParamsWifiEnterpriseEapPeap as ConfigurationCreateParamsWifiEnterpriseEapPeap,
        ConfigurationCreateParamsWifiEnterpriseEapTls as ConfigurationCreateParamsWifiEnterpriseEapTls,
        ConfigurationCreateParamsWifiPersonalPsk as ConfigurationCreateParamsWifiPersonalPsk,
    )
    from stripe.params.terminal._configuration_delete_params import (
        ConfigurationDeleteParams as ConfigurationDeleteParams,
    )
    from stripe.params.terminal._configuration_list_params import (
        ConfigurationListParams as ConfigurationListParams,
    )
    from stripe.params.terminal._configuration_modify_params import (
        ConfigurationModifyParams as ConfigurationModifyParams,
        ConfigurationModifyParamsBbposWisepad3 as ConfigurationModifyParamsBbposWisepad3,
        ConfigurationModifyParamsBbposWiseposE as ConfigurationModifyParamsBbposWiseposE,
        ConfigurationModifyParamsOffline as ConfigurationModifyParamsOffline,
        ConfigurationModifyParamsRebootWindow as ConfigurationModifyParamsRebootWindow,
        ConfigurationModifyParamsStripeS700 as ConfigurationModifyParamsStripeS700,
        ConfigurationModifyParamsTipping as ConfigurationModifyParamsTipping,
        ConfigurationModifyParamsTippingAed as ConfigurationModifyParamsTippingAed,
        ConfigurationModifyParamsTippingAud as ConfigurationModifyParamsTippingAud,
        ConfigurationModifyParamsTippingBgn as ConfigurationModifyParamsTippingBgn,
        ConfigurationModifyParamsTippingCad as ConfigurationModifyParamsTippingCad,
        ConfigurationModifyParamsTippingChf as ConfigurationModifyParamsTippingChf,
        ConfigurationModifyParamsTippingCzk as ConfigurationModifyParamsTippingCzk,
        ConfigurationModifyParamsTippingDkk as ConfigurationModifyParamsTippingDkk,
        ConfigurationModifyParamsTippingEur as ConfigurationModifyParamsTippingEur,
        ConfigurationModifyParamsTippingGbp as ConfigurationModifyParamsTippingGbp,
        ConfigurationModifyParamsTippingGip as ConfigurationModifyParamsTippingGip,
        ConfigurationModifyParamsTippingHkd as ConfigurationModifyParamsTippingHkd,
        ConfigurationModifyParamsTippingHuf as ConfigurationModifyParamsTippingHuf,
        ConfigurationModifyParamsTippingJpy as ConfigurationModifyParamsTippingJpy,
        ConfigurationModifyParamsTippingMxn as ConfigurationModifyParamsTippingMxn,
        ConfigurationModifyParamsTippingMyr as ConfigurationModifyParamsTippingMyr,
        ConfigurationModifyParamsTippingNok as ConfigurationModifyParamsTippingNok,
        ConfigurationModifyParamsTippingNzd as ConfigurationModifyParamsTippingNzd,
        ConfigurationModifyParamsTippingPln as ConfigurationModifyParamsTippingPln,
        ConfigurationModifyParamsTippingRon as ConfigurationModifyParamsTippingRon,
        ConfigurationModifyParamsTippingSek as ConfigurationModifyParamsTippingSek,
        ConfigurationModifyParamsTippingSgd as ConfigurationModifyParamsTippingSgd,
        ConfigurationModifyParamsTippingUsd as ConfigurationModifyParamsTippingUsd,
        ConfigurationModifyParamsVerifoneP400 as ConfigurationModifyParamsVerifoneP400,
        ConfigurationModifyParamsWifi as ConfigurationModifyParamsWifi,
        ConfigurationModifyParamsWifiEnterpriseEapPeap as ConfigurationModifyParamsWifiEnterpriseEapPeap,
        ConfigurationModifyParamsWifiEnterpriseEapTls as ConfigurationModifyParamsWifiEnterpriseEapTls,
        ConfigurationModifyParamsWifiPersonalPsk as ConfigurationModifyParamsWifiPersonalPsk,
    )
    from stripe.params.terminal._configuration_retrieve_params import (
        ConfigurationRetrieveParams as ConfigurationRetrieveParams,
    )
    from stripe.params.terminal._configuration_update_params import (
        ConfigurationUpdateParams as ConfigurationUpdateParams,
        ConfigurationUpdateParamsBbposWisepad3 as ConfigurationUpdateParamsBbposWisepad3,
        ConfigurationUpdateParamsBbposWiseposE as ConfigurationUpdateParamsBbposWiseposE,
        ConfigurationUpdateParamsOffline as ConfigurationUpdateParamsOffline,
        ConfigurationUpdateParamsRebootWindow as ConfigurationUpdateParamsRebootWindow,
        ConfigurationUpdateParamsStripeS700 as ConfigurationUpdateParamsStripeS700,
        ConfigurationUpdateParamsTipping as ConfigurationUpdateParamsTipping,
        ConfigurationUpdateParamsTippingAed as ConfigurationUpdateParamsTippingAed,
        ConfigurationUpdateParamsTippingAud as ConfigurationUpdateParamsTippingAud,
        ConfigurationUpdateParamsTippingBgn as ConfigurationUpdateParamsTippingBgn,
        ConfigurationUpdateParamsTippingCad as ConfigurationUpdateParamsTippingCad,
        ConfigurationUpdateParamsTippingChf as ConfigurationUpdateParamsTippingChf,
        ConfigurationUpdateParamsTippingCzk as ConfigurationUpdateParamsTippingCzk,
        ConfigurationUpdateParamsTippingDkk as ConfigurationUpdateParamsTippingDkk,
        ConfigurationUpdateParamsTippingEur as ConfigurationUpdateParamsTippingEur,
        ConfigurationUpdateParamsTippingGbp as ConfigurationUpdateParamsTippingGbp,
        ConfigurationUpdateParamsTippingGip as ConfigurationUpdateParamsTippingGip,
        ConfigurationUpdateParamsTippingHkd as ConfigurationUpdateParamsTippingHkd,
        ConfigurationUpdateParamsTippingHuf as ConfigurationUpdateParamsTippingHuf,
        ConfigurationUpdateParamsTippingJpy as ConfigurationUpdateParamsTippingJpy,
        ConfigurationUpdateParamsTippingMxn as ConfigurationUpdateParamsTippingMxn,
        ConfigurationUpdateParamsTippingMyr as ConfigurationUpdateParamsTippingMyr,
        ConfigurationUpdateParamsTippingNok as ConfigurationUpdateParamsTippingNok,
        ConfigurationUpdateParamsTippingNzd as ConfigurationUpdateParamsTippingNzd,
        ConfigurationUpdateParamsTippingPln as ConfigurationUpdateParamsTippingPln,
        ConfigurationUpdateParamsTippingRon as ConfigurationUpdateParamsTippingRon,
        ConfigurationUpdateParamsTippingSek as ConfigurationUpdateParamsTippingSek,
        ConfigurationUpdateParamsTippingSgd as ConfigurationUpdateParamsTippingSgd,
        ConfigurationUpdateParamsTippingUsd as ConfigurationUpdateParamsTippingUsd,
        ConfigurationUpdateParamsVerifoneP400 as ConfigurationUpdateParamsVerifoneP400,
        ConfigurationUpdateParamsWifi as ConfigurationUpdateParamsWifi,
        ConfigurationUpdateParamsWifiEnterpriseEapPeap as ConfigurationUpdateParamsWifiEnterpriseEapPeap,
        ConfigurationUpdateParamsWifiEnterpriseEapTls as ConfigurationUpdateParamsWifiEnterpriseEapTls,
        ConfigurationUpdateParamsWifiPersonalPsk as ConfigurationUpdateParamsWifiPersonalPsk,
    )
    from stripe.params.terminal._connection_token_create_params import (
        ConnectionTokenCreateParams as ConnectionTokenCreateParams,
    )
    from stripe.params.terminal._location_create_params import (
        LocationCreateParams as LocationCreateParams,
        LocationCreateParamsAddress as LocationCreateParamsAddress,
        LocationCreateParamsAddressKana as LocationCreateParamsAddressKana,
        LocationCreateParamsAddressKanji as LocationCreateParamsAddressKanji,
    )
    from stripe.params.terminal._location_delete_params import (
        LocationDeleteParams as LocationDeleteParams,
    )
    from stripe.params.terminal._location_list_params import (
        LocationListParams as LocationListParams,
    )
    from stripe.params.terminal._location_modify_params import (
        LocationModifyParams as LocationModifyParams,
        LocationModifyParamsAddress as LocationModifyParamsAddress,
        LocationModifyParamsAddressKana as LocationModifyParamsAddressKana,
        LocationModifyParamsAddressKanji as LocationModifyParamsAddressKanji,
    )
    from stripe.params.terminal._location_retrieve_params import (
        LocationRetrieveParams as LocationRetrieveParams,
    )
    from stripe.params.terminal._location_update_params import (
        LocationUpdateParams as LocationUpdateParams,
        LocationUpdateParamsAddress as LocationUpdateParamsAddress,
        LocationUpdateParamsAddressKana as LocationUpdateParamsAddressKana,
        LocationUpdateParamsAddressKanji as LocationUpdateParamsAddressKanji,
    )
    from stripe.params.terminal._onboarding_link_create_params import (
        OnboardingLinkCreateParams as OnboardingLinkCreateParams,
        OnboardingLinkCreateParamsLinkOptions as OnboardingLinkCreateParamsLinkOptions,
        OnboardingLinkCreateParamsLinkOptionsAppleTermsAndConditions as OnboardingLinkCreateParamsLinkOptionsAppleTermsAndConditions,
    )
    from stripe.params.terminal._reader_cancel_action_params import (
        ReaderCancelActionParams as ReaderCancelActionParams,
    )
    from stripe.params.terminal._reader_collect_inputs_params import (
        ReaderCollectInputsParams as ReaderCollectInputsParams,
        ReaderCollectInputsParamsInput as ReaderCollectInputsParamsInput,
        ReaderCollectInputsParamsInputCustomText as ReaderCollectInputsParamsInputCustomText,
        ReaderCollectInputsParamsInputSelection as ReaderCollectInputsParamsInputSelection,
        ReaderCollectInputsParamsInputSelectionChoice as ReaderCollectInputsParamsInputSelectionChoice,
        ReaderCollectInputsParamsInputToggle as ReaderCollectInputsParamsInputToggle,
    )
    from stripe.params.terminal._reader_collect_payment_method_params import (
        ReaderCollectPaymentMethodParams as ReaderCollectPaymentMethodParams,
        ReaderCollectPaymentMethodParamsCollectConfig as ReaderCollectPaymentMethodParamsCollectConfig,
        ReaderCollectPaymentMethodParamsCollectConfigTipping as ReaderCollectPaymentMethodParamsCollectConfigTipping,
    )
    from stripe.params.terminal._reader_confirm_payment_intent_params import (
        ReaderConfirmPaymentIntentParams as ReaderConfirmPaymentIntentParams,
        ReaderConfirmPaymentIntentParamsConfirmConfig as ReaderConfirmPaymentIntentParamsConfirmConfig,
    )
    from stripe.params.terminal._reader_create_params import (
        ReaderCreateParams as ReaderCreateParams,
    )
    from stripe.params.terminal._reader_delete_params import (
        ReaderDeleteParams as ReaderDeleteParams,
    )
    from stripe.params.terminal._reader_list_params import (
        ReaderListParams as ReaderListParams,
    )
    from stripe.params.terminal._reader_modify_params import (
        ReaderModifyParams as ReaderModifyParams,
    )
    from stripe.params.terminal._reader_present_payment_method_params import (
        ReaderPresentPaymentMethodParams as ReaderPresentPaymentMethodParams,
        ReaderPresentPaymentMethodParamsCard as ReaderPresentPaymentMethodParamsCard,
        ReaderPresentPaymentMethodParamsCardPresent as ReaderPresentPaymentMethodParamsCardPresent,
        ReaderPresentPaymentMethodParamsInteracPresent as ReaderPresentPaymentMethodParamsInteracPresent,
    )
    from stripe.params.terminal._reader_process_payment_intent_params import (
        ReaderProcessPaymentIntentParams as ReaderProcessPaymentIntentParams,
        ReaderProcessPaymentIntentParamsProcessConfig as ReaderProcessPaymentIntentParamsProcessConfig,
        ReaderProcessPaymentIntentParamsProcessConfigTipping as ReaderProcessPaymentIntentParamsProcessConfigTipping,
    )
    from stripe.params.terminal._reader_process_setup_intent_params import (
        ReaderProcessSetupIntentParams as ReaderProcessSetupIntentParams,
        ReaderProcessSetupIntentParamsProcessConfig as ReaderProcessSetupIntentParamsProcessConfig,
    )
    from stripe.params.terminal._reader_refund_payment_params import (
        ReaderRefundPaymentParams as ReaderRefundPaymentParams,
        ReaderRefundPaymentParamsRefundPaymentConfig as ReaderRefundPaymentParamsRefundPaymentConfig,
    )
    from stripe.params.terminal._reader_retrieve_params import (
        ReaderRetrieveParams as ReaderRetrieveParams,
    )
    from stripe.params.terminal._reader_set_reader_display_params import (
        ReaderSetReaderDisplayParams as ReaderSetReaderDisplayParams,
        ReaderSetReaderDisplayParamsCart as ReaderSetReaderDisplayParamsCart,
        ReaderSetReaderDisplayParamsCartLineItem as ReaderSetReaderDisplayParamsCartLineItem,
    )
    from stripe.params.terminal._reader_succeed_input_collection_params import (
        ReaderSucceedInputCollectionParams as ReaderSucceedInputCollectionParams,
    )
    from stripe.params.terminal._reader_timeout_input_collection_params import (
        ReaderTimeoutInputCollectionParams as ReaderTimeoutInputCollectionParams,
    )
    from stripe.params.terminal._reader_update_params import (
        ReaderUpdateParams as ReaderUpdateParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "ConfigurationCreateParams": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsBbposWisepad3": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsBbposWiseposE": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsOffline": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsRebootWindow": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsStripeS700": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTipping": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingAed": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingAud": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingBgn": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingCad": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingChf": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingCzk": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingDkk": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingEur": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingGbp": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingGip": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingHkd": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingHuf": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingJpy": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingMxn": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingMyr": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingNok": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingNzd": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingPln": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingRon": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingSek": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingSgd": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsTippingUsd": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsVerifoneP400": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsWifi": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsWifiEnterpriseEapPeap": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsWifiEnterpriseEapTls": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationCreateParamsWifiPersonalPsk": (
        "stripe.params.terminal._configuration_create_params",
        False,
    ),
    "ConfigurationDeleteParams": (
        "stripe.params.terminal._configuration_delete_params",
        False,
    ),
    "ConfigurationListParams": (
        "stripe.params.terminal._configuration_list_params",
        False,
    ),
    "ConfigurationModifyParams": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsBbposWisepad3": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsBbposWiseposE": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsOffline": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsRebootWindow": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsStripeS700": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTipping": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingAed": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingAud": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingBgn": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingCad": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingChf": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingCzk": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingDkk": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingEur": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingGbp": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingGip": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingHkd": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingHuf": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingJpy": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingMxn": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingMyr": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingNok": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingNzd": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingPln": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingRon": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingSek": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingSgd": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsTippingUsd": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsVerifoneP400": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsWifi": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsWifiEnterpriseEapPeap": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsWifiEnterpriseEapTls": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationModifyParamsWifiPersonalPsk": (
        "stripe.params.terminal._configuration_modify_params",
        False,
    ),
    "ConfigurationRetrieveParams": (
        "stripe.params.terminal._configuration_retrieve_params",
        False,
    ),
    "ConfigurationUpdateParams": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsBbposWisepad3": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsBbposWiseposE": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsOffline": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsRebootWindow": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsStripeS700": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTipping": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingAed": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingAud": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingBgn": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingCad": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingChf": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingCzk": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingDkk": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingEur": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingGbp": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingGip": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingHkd": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingHuf": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingJpy": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingMxn": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingMyr": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingNok": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingNzd": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingPln": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingRon": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingSek": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingSgd": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsTippingUsd": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsVerifoneP400": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsWifi": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsWifiEnterpriseEapPeap": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsWifiEnterpriseEapTls": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConfigurationUpdateParamsWifiPersonalPsk": (
        "stripe.params.terminal._configuration_update_params",
        False,
    ),
    "ConnectionTokenCreateParams": (
        "stripe.params.terminal._connection_token_create_params",
        False,
    ),
    "LocationCreateParams": (
        "stripe.params.terminal._location_create_params",
        False,
    ),
    "LocationCreateParamsAddress": (
        "stripe.params.terminal._location_create_params",
        False,
    ),
    "LocationCreateParamsAddressKana": (
        "stripe.params.terminal._location_create_params",
        False,
    ),
    "LocationCreateParamsAddressKanji": (
        "stripe.params.terminal._location_create_params",
        False,
    ),
    "LocationDeleteParams": (
        "stripe.params.terminal._location_delete_params",
        False,
    ),
    "LocationListParams": (
        "stripe.params.terminal._location_list_params",
        False,
    ),
    "LocationModifyParams": (
        "stripe.params.terminal._location_modify_params",
        False,
    ),
    "LocationModifyParamsAddress": (
        "stripe.params.terminal._location_modify_params",
        False,
    ),
    "LocationModifyParamsAddressKana": (
        "stripe.params.terminal._location_modify_params",
        False,
    ),
    "LocationModifyParamsAddressKanji": (
        "stripe.params.terminal._location_modify_params",
        False,
    ),
    "LocationRetrieveParams": (
        "stripe.params.terminal._location_retrieve_params",
        False,
    ),
    "LocationUpdateParams": (
        "stripe.params.terminal._location_update_params",
        False,
    ),
    "LocationUpdateParamsAddress": (
        "stripe.params.terminal._location_update_params",
        False,
    ),
    "LocationUpdateParamsAddressKana": (
        "stripe.params.terminal._location_update_params",
        False,
    ),
    "LocationUpdateParamsAddressKanji": (
        "stripe.params.terminal._location_update_params",
        False,
    ),
    "OnboardingLinkCreateParams": (
        "stripe.params.terminal._onboarding_link_create_params",
        False,
    ),
    "OnboardingLinkCreateParamsLinkOptions": (
        "stripe.params.terminal._onboarding_link_create_params",
        False,
    ),
    "OnboardingLinkCreateParamsLinkOptionsAppleTermsAndConditions": (
        "stripe.params.terminal._onboarding_link_create_params",
        False,
    ),
    "ReaderCancelActionParams": (
        "stripe.params.terminal._reader_cancel_action_params",
        False,
    ),
    "ReaderCollectInputsParams": (
        "stripe.params.terminal._reader_collect_inputs_params",
        False,
    ),
    "ReaderCollectInputsParamsInput": (
        "stripe.params.terminal._reader_collect_inputs_params",
        False,
    ),
    "ReaderCollectInputsParamsInputCustomText": (
        "stripe.params.terminal._reader_collect_inputs_params",
        False,
    ),
    "ReaderCollectInputsParamsInputSelection": (
        "stripe.params.terminal._reader_collect_inputs_params",
        False,
    ),
    "ReaderCollectInputsParamsInputSelectionChoice": (
        "stripe.params.terminal._reader_collect_inputs_params",
        False,
    ),
    "ReaderCollectInputsParamsInputToggle": (
        "stripe.params.terminal._reader_collect_inputs_params",
        False,
    ),
    "ReaderCollectPaymentMethodParams": (
        "stripe.params.terminal._reader_collect_payment_method_params",
        False,
    ),
    "ReaderCollectPaymentMethodParamsCollectConfig": (
        "stripe.params.terminal._reader_collect_payment_method_params",
        False,
    ),
    "ReaderCollectPaymentMethodParamsCollectConfigTipping": (
        "stripe.params.terminal._reader_collect_payment_method_params",
        False,
    ),
    "ReaderConfirmPaymentIntentParams": (
        "stripe.params.terminal._reader_confirm_payment_intent_params",
        False,
    ),
    "ReaderConfirmPaymentIntentParamsConfirmConfig": (
        "stripe.params.terminal._reader_confirm_payment_intent_params",
        False,
    ),
    "ReaderCreateParams": (
        "stripe.params.terminal._reader_create_params",
        False,
    ),
    "ReaderDeleteParams": (
        "stripe.params.terminal._reader_delete_params",
        False,
    ),
    "ReaderListParams": ("stripe.params.terminal._reader_list_params", False),
    "ReaderModifyParams": (
        "stripe.params.terminal._reader_modify_params",
        False,
    ),
    "ReaderPresentPaymentMethodParams": (
        "stripe.params.terminal._reader_present_payment_method_params",
        False,
    ),
    "ReaderPresentPaymentMethodParamsCard": (
        "stripe.params.terminal._reader_present_payment_method_params",
        False,
    ),
    "ReaderPresentPaymentMethodParamsCardPresent": (
        "stripe.params.terminal._reader_present_payment_method_params",
        False,
    ),
    "ReaderPresentPaymentMethodParamsInteracPresent": (
        "stripe.params.terminal._reader_present_payment_method_params",
        False,
    ),
    "ReaderProcessPaymentIntentParams": (
        "stripe.params.terminal._reader_process_payment_intent_params",
        False,
    ),
    "ReaderProcessPaymentIntentParamsProcessConfig": (
        "stripe.params.terminal._reader_process_payment_intent_params",
        False,
    ),
    "ReaderProcessPaymentIntentParamsProcessConfigTipping": (
        "stripe.params.terminal._reader_process_payment_intent_params",
        False,
    ),
    "ReaderProcessSetupIntentParams": (
        "stripe.params.terminal._reader_process_setup_intent_params",
        False,
    ),
    "ReaderProcessSetupIntentParamsProcessConfig": (
        "stripe.params.terminal._reader_process_setup_intent_params",
        False,
    ),
    "ReaderRefundPaymentParams": (
        "stripe.params.terminal._reader_refund_payment_params",
        False,
    ),
    "ReaderRefundPaymentParamsRefundPaymentConfig": (
        "stripe.params.terminal._reader_refund_payment_params",
        False,
    ),
    "ReaderRetrieveParams": (
        "stripe.params.terminal._reader_retrieve_params",
        False,
    ),
    "ReaderSetReaderDisplayParams": (
        "stripe.params.terminal._reader_set_reader_display_params",
        False,
    ),
    "ReaderSetReaderDisplayParamsCart": (
        "stripe.params.terminal._reader_set_reader_display_params",
        False,
    ),
    "ReaderSetReaderDisplayParamsCartLineItem": (
        "stripe.params.terminal._reader_set_reader_display_params",
        False,
    ),
    "ReaderSucceedInputCollectionParams": (
        "stripe.params.terminal._reader_succeed_input_collection_params",
        False,
    ),
    "ReaderTimeoutInputCollectionParams": (
        "stripe.params.terminal._reader_timeout_input_collection_params",
        False,
    ),
    "ReaderUpdateParams": (
        "stripe.params.terminal._reader_update_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from importlib import import_module
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.entitlements._active_entitlement_list_params import (
        ActiveEntitlementListParams as ActiveEntitlementListParams,
    )
    from stripe.params.entitlements._active_entitlement_retrieve_params import (
        ActiveEntitlementRetrieveParams as ActiveEntitlementRetrieveParams,
    )
    from stripe.params.entitlements._feature_create_params import (
        FeatureCreateParams as FeatureCreateParams,
    )
    from stripe.params.entitlements._feature_list_params import (
        FeatureListParams as FeatureListParams,
    )
    from stripe.params.entitlements._feature_modify_params import (
        FeatureModifyParams as FeatureModifyParams,
    )
    from stripe.params.entitlements._feature_retrieve_params import (
        FeatureRetrieveParams as FeatureRetrieveParams,
    )
    from stripe.params.entitlements._feature_update_params import (
        FeatureUpdateParams as FeatureUpdateParams,
    )

# name -> (import_target, is_submodule)
_import_map = {
    "ActiveEntitlementListParams": (
        "stripe.params.entitlements._active_entitlement_list_params",
        False,
    ),
    "ActiveEntitlementRetrieveParams": (
        "stripe.params.entitlements._active_entitlement_retrieve_params",
        False,
    ),
    "FeatureCreateParams": (
        "stripe.params.entitlements._feature_create_params",
        False,
    ),
    "FeatureListParams": (
        "stripe.params.entitlements._feature_list_params",
        False,
    ),
    "FeatureModifyParams": (
        "stripe.params.entitlements._feature_modify_params",
        False,
    ),
    "FeatureRetrieveParams": (
        "stripe.params.entitlements._feature_retrieve_params",
        False,
    ),
    "FeatureUpdateParams": (
        "stripe.params.entitlements._feature_update_params",
        False,
    ),
}
if not TYPE_CHECKING:

    def __getattr__(name):
        try:
            target, is_submodule = _import_map[name]
            module = import_module(target)
            if is_submodule:
                return module

            return getattr(
                module,
                name,
            )
        except KeyError:
            raise AttributeError()



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params.radar._early_fraud_warning_list_params import (
        EarlyFraudWarningListParams,
    )
    from stripe.params.radar._early_fraud_warning_retrieve_params import (
        EarlyFraudWarningRetrieveParams,
    )
    from stripe.radar._early_fraud_warning import EarlyFraudWarning


class EarlyFraudWarningService(StripeService):
    def list(
        self,
        params: Optional["EarlyFraudWarningListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[EarlyFraudWarning]":
        """
        Returns a list of early fraud warnings.
        """
        return cast(
            "ListObject[EarlyFraudWarning]",
            self._request(
                "get",
                "/v1/radar/early_fraud_warnings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["EarlyFraudWarningListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[EarlyFraudWarning]":
        """
        Returns a list of early fraud warnings.
        """
        return cast(
            "ListObject[EarlyFraudWarning]",
            await self._request_async(
                "get",
                "/v1/radar/early_fraud_warnings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        early_fraud_warning: str,
        params: Optional["EarlyFraudWarningRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "EarlyFraudWarning":
        """
        Retrieves the details of an early fraud warning that has previously been created.

        Please refer to the [early fraud warning](https://docs.stripe.com/api#early_fraud_warning_object) object reference for more details.
        """
        return cast(
            "EarlyFraudWarning",
            self._request(
                "get",
                "/v1/radar/early_fraud_warnings/{early_fraud_warning}".format(
                    early_fraud_warning=sanitize_id(early_fraud_warning),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        early_fraud_warning: str,
        params: Optional["EarlyFraudWarningRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "EarlyFraudWarning":
        """
        Retrieves the details of an early fraud warning that has previously been created.

        Please refer to the [early fraud warning](https://docs.stripe.com/api#early_fraud_warning_object) object reference for more details.
        """
        return cast(
            "EarlyFraudWarning",
            await self._request_async(
                "get",
                "/v1/radar/early_fraud_warnings/{early_fraud_warning}".format(
                    early_fraud_warning=sanitize_id(early_fraud_warning),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



import datetime
import json
import tempfile
import uuid
from collections import OrderedDict
from urllib.parse import urlencode, urlsplit

import pytest
import urllib3

import stripe
import io
from stripe._api_requestor import _api_encode, _APIRequestor
from stripe._customer import Customer
from stripe._request_options import RequestOptions
from stripe._requestor_options import (
    RequestorOptions,
    _GlobalRequestorOptions,
)
from stripe._stripe_object import StripeObject
from stripe._stripe_response import (
    StripeStreamResponse,
    StripeStreamResponseAsync,
)
from stripe.v2._deleted_object import DeletedObject
from tests.http_client_mock import HTTPClientMock

VALID_API_METHODS = ("get", "post", "delete")


class GMT1(datetime.tzinfo):
    def utcoffset(self, dt):
        return datetime.timedelta(hours=1)

    def dst(self, dt):
        return datetime.timedelta(0)

    def tzname(self, dt):
        return "Europe/Prague"


class AnyUUID4Matcher(object):
    def __eq__(self, other):
        try:
            uuid.UUID(other, version=4)
        except ValueError:
            return False
        return True

    def __repr__(self):
        return "AnyUUID4Matcher()"


class IsNoneMatcher:
    """
    Matcher to make assertions against None because `assert_requested` doesn't
    run checks if you pass `None` as the expected value.
    """

    def __eq__(self, other):
        return other is None

    def __repr__(self):
        return "None (from IsNoneMatcher())"


class TestAPIRequestor(object):
    ENCODE_INPUTS = {
        "dict": {
            "astring": "bar",
            "anint": 5,
            "anull": None,
            "adatetime": datetime.datetime(2013, 1, 1, tzinfo=GMT1()),
            "atuple": (1, 2),
            "adict": {"foo": "bar", "boz": 5},
            "alist": ["foo", "bar"],
        },
        "list": [1, "foo", "baz"],
        "string": "boo",
        "unicode": "\u1234",
        "datetime": datetime.datetime(2013, 1, 1, second=1, tzinfo=GMT1()),
        "none": None,
    }

    ENCODE_EXPECTATIONS = {
        "dict": [
            ("%s[astring]", "bar"),
            ("%s[anint]", 5),
            ("%s[adatetime]", 1356994800),
            ("%s[adict][foo]", "bar"),
            ("%s[adict][boz]", 5),
            ("%s[alist][0]", "foo"),
            ("%s[alist][1]", "bar"),
            ("%s[atuple][0]", 1),
            ("%s[atuple][1]", 2),
        ],
        "list": [("%s[0]", 1), ("%s[1]", "foo"), ("%s[2]", "baz")],
        "string": [("%s", "boo")],
        "unicode": [("%s", "\u1234")],
        "datetime": [("%s", 1356994801)],
        "none": [],
    }

    @pytest.fixture(autouse=True)
    def setup_stripe(self):
        orig_attrs = {
            "api_key": stripe.api_key,
            "api_version": stripe.api_version,
            "default_http_client": stripe.default_http_client,
            "enable_telemetry": stripe.enable_telemetry,
        }
        stripe.api_key = "sk_test_123"
        stripe.api_version = "2017-12-14"
        stripe.default_http_client = None
        stripe.enable_telemetry = False
        yield
        stripe.api_key = orig_attrs["api_key"]
        stripe.api_version = orig_attrs["api_version"]
        stripe.default_http_client = orig_attrs["default_http_client"]
        stripe.enable_telemetry = orig_attrs["enable_telemetry"]

    @pytest.fixture
    def requestor(self, http_client_mock):
        requestor = _APIRequestor(
            client=http_client_mock.get_mock_http_client(),
            options=_GlobalRequestorOptions(),
        )
        return requestor

    @property
    def v1_path(self):
        return "/v1/foo"

    @property
    def v2_path(self):
        return "/v2/foo"

    def encoder_check(self, key):
        stk_key = "my%s" % (key,)

        value = self.ENCODE_INPUTS[key]
        expectation = [
            (k % (stk_key,), v) for k, v in self.ENCODE_EXPECTATIONS[key]
        ]

        stk = []
        fn = getattr(_APIRequestor, "encode_%s" % (key,))
        fn(stk, stk_key, value)

        if isinstance(value, dict):
            expectation.sort()
            stk.sort()

        assert stk == expectation, stk

    def _test_encode_naive_datetime(self):
        stk = []

        _APIRequestor.encode_datetime(
            stk, "test", datetime.datetime(2013, 1, 1)
        )

        # Naive datetimes will encode differently depending on your system
        # local time.  Since we don't know the local time of your system,
        # we just check that naive encodings are within 24 hours of correct.
        assert abs(stk[0][1] - 1356994800) <= 60 * 60 * 24

    def test_param_encoding(self, requestor, http_client_mock):
        expectation = []
        for type_, values in iter(self.ENCODE_EXPECTATIONS.items()):
            expectation.extend([(k % (type_,), str(v)) for k, v in values])

        query_string = (
            urlencode(expectation).replace("%5B", "[").replace("%5D", "]")
        )
        http_client_mock.stub_request(
            "get", query_string=query_string, rbody="{}", rcode=200
        )

        requestor.request("get", "", self.ENCODE_INPUTS, base_address="api")

        http_client_mock.assert_requested("get", query_string=query_string)

    def test_param_api_mode_preview(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "post", path=self.v2_path, rbody="{}", rcode=200
        )

        requestor.request(
            "post", self.v2_path, self.ENCODE_INPUTS, base_address="api"
        )

        expectation = '{"dict": {"astring": "bar", "anint": 5, "anull": null, "adatetime": 1356994800, "atuple": [1, 2], "adict": {"foo": "bar", "boz": 5}, "alist": ["foo", "bar"]}, "list": [1, "foo", "baz"], "string": "boo", "unicode": "\\u1234", "datetime": 1356994801, "none": null}'

        http_client_mock.assert_requested(
            "post",
            content_type="application/json",
            post_data=expectation,
            is_json=True,
        )

    def test_encodes_null_values_preview(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "post", path=self.v2_path, rbody="{}", rcode=200
        )

        requestor.request(
            "post",
            self.v2_path,
            {"foo": None},
            base_address="api",
        )

        http_client_mock.assert_requested(
            "post",
            content_type="application/json",
            post_data='{"foo": null}',
            is_json=True,
        )

    def test_dictionary_list_encoding(self):
        params = {"foo": {"0": {"bar": "bat"}}}
        encoded = list(_api_encode(params))
        key, value = encoded[0]

        assert key == "foo[0][bar]"
        assert value == "bat"

    def test_ordereddict_encoding(self):
        params = {
            "ordered": OrderedDict(
                [
                    ("one", 1),
                    ("two", 2),
                    ("three", 3),
                    ("nested", OrderedDict([("a", "a"), ("b", "b")])),
                ]
            )
        }
        encoded = list(_api_encode(params))

        assert encoded[0][0] == "ordered[one]"
        assert encoded[1][0] == "ordered[two]"
        assert encoded[2][0] == "ordered[three]"
        assert encoded[3][0] == "ordered[nested][a]"
        assert encoded[4][0] == "ordered[nested][b]"

    def test_url_construction(self, requestor, http_client_mock):
        CASES = (
            (f"{stripe.api_base}?foo=bar", "", {"foo": "bar"}),
            (f"{stripe.api_base}?foo=bar", "?", {"foo": "bar"}),
            (stripe.api_base, "", {}),
            (
                f"{stripe.api_base}/%20spaced?baz=5&foo=bar%24",
                "/%20spaced?foo=bar%24",
                {"baz": "5"},
            ),
            # duplicate query params keys should be deduped
            (
                f"{stripe.api_base}?foo=bar",
                "?foo=bar",
                {"foo": "bar"},
            ),
        )

        for expected, url, params in CASES:
            path = urlsplit(expected).path
            query_string = urlsplit(expected).query
            http_client_mock.stub_request(
                "get",
                path=path,
                query_string=query_string,
                rbody="{}",
                rcode=200,
            )

            requestor.request("get", url, params, base_address="api")

            http_client_mock.assert_requested("get", abs_url=expected)

    def test_empty_methods(self, requestor, http_client_mock):
        for meth in VALID_API_METHODS:
            http_client_mock.stub_request(
                meth, path=self.v1_path, rbody="{}", rcode=200
            )

            resp = requestor.request(
                meth, self.v1_path, {}, base_address="api"
            )

            if meth == "post":
                post_data = ""
            else:
                post_data = None

            http_client_mock.assert_requested(meth, post_data=post_data)
            assert isinstance(resp, StripeObject)

            assert resp == {}

    @pytest.mark.anyio
    async def test_empty_methods_async(self, requestor, http_client_mock):
        for meth in VALID_API_METHODS:
            http_client_mock.stub_request(
                meth,
                path=self.v1_path,
                rbody="{}",
                rcode=200,
            )

            resp = await requestor.request_async(
                meth, self.v1_path, {}, base_address="api"
            )

            if meth == "post":
                post_data = ""
            else:
                post_data = None

            http_client_mock.assert_requested(meth, post_data=post_data)
            assert isinstance(resp, StripeObject)

            assert resp == {}

    @pytest.mark.anyio
    async def test_empty_methods_streaming_response_async(
        self, requestor, http_client_mock
    ):
        async def async_iter():
            yield b"this"
            yield b"is"
            yield b"data"

        for meth in VALID_API_METHODS:
            http_client_mock.stub_request(
                meth,
                path=self.v1_path,
                rbody=async_iter(),
                rcode=200,
            )

            resp = await requestor.request_stream_async(
                meth,
                self.v1_path,
                {},
                base_address="api",
            )

            if meth == "post":
                post_data = ""
            else:
                post_data = None

            http_client_mock.assert_requested(meth, post_data=post_data)
            assert isinstance(resp, StripeStreamResponseAsync)

            assert b"".join([x async for x in resp.stream()]) == b"thisisdata"

    def test_empty_methods_streaming_response(
        self, requestor, http_client_mock
    ):
        for meth in VALID_API_METHODS:
            http_client_mock.stub_request(
                meth,
                path=self.v1_path,
                rbody=io.BytesIO(b"thisisdata"),
                rcode=200,
            )

            resp = requestor.request_stream(
                meth,
                self.v1_path,
                {},
                base_address="api",
            )

            if meth == "post":
                post_data = ""
            else:
                post_data = None

            http_client_mock.assert_requested(meth, post_data=post_data)
            assert isinstance(resp, StripeStreamResponse)

            assert resp.io.getvalue() == b"thisisdata"

    def test_methods_with_params_and_response(
        self, requestor, http_client_mock
    ):
        for method in VALID_API_METHODS:
            encoded = (
                "adict[frobble]=bits&adatetime=1356994800&"
                "alist[0]=1&alist[1]=2&alist[2]=3"
            )

            http_client_mock.stub_request(
                method,
                path=self.v1_path,
                query_string=encoded if method != "post" else "",
                rbody='{"foo": "bar", "baz": 6}',
                rcode=200,
            )

            params = {
                "alist": [1, 2, 3],
                "adict": {"frobble": "bits"},
                "adatetime": datetime.datetime(2013, 1, 1, tzinfo=GMT1()),
            }

            resp = requestor.request(
                method,
                self.v1_path,
                params,
                base_address="api",
            )
            assert isinstance(resp, StripeObject)

            assert resp == {"foo": "bar", "baz": 6}

            if method == "post":
                http_client_mock.assert_requested(
                    method,
                    post_data=encoded,
                )
            else:
                abs_url = "%s%s?%s" % (
                    stripe.api_base,
                    self.v1_path,
                    encoded,
                )
                http_client_mock.assert_requested(method, abs_url=abs_url)

    def test_methods_with_params_and_streaming_response(
        self, requestor, http_client_mock
    ):
        for method in VALID_API_METHODS:
            encoded = (
                "adict[frobble]=bits&adatetime=1356994800&"
                "alist[0]=1&alist[1]=2&alist[2]=3"
            )

            http_client_mock.stub_request(
                method,
                path=self.v1_path,
                query_string=encoded if method != "post" else "",
                rbody=io.BytesIO(b'{"foo": "bar", "baz": 6}'),
                rcode=200,
            )

            params = {
                "alist": [1, 2, 3],
                "adict": {"frobble": "bits"},
                "adatetime": datetime.datetime(2013, 1, 1, tzinfo=GMT1()),
            }

            resp = requestor.request_stream(
                method,
                self.v1_path,
                params,
                base_address="api",
            )
            assert isinstance(resp, StripeStreamResponse)

            assert resp.io.getvalue() == b'{"foo": "bar", "baz": 6}'

            if method == "post":
                http_client_mock.assert_requested(method, post_data=encoded)
            else:
                abs_url = "%s%s?%s" % (
                    stripe.api_base,
                    self.v1_path,
                    encoded,
                )
                http_client_mock.assert_requested(method, abs_url=abs_url)

    def test_delete_methods(self, requestor, http_client_mock):
        for path in [self.v1_path, self.v2_path]:
            method = "delete"
            http_client_mock.stub_request(
                method,
                path=path,
                rbody=json.dumps({"id": "abc_123", "object": "customer"}),
                rcode=200,
            )

            resp = requestor.request(method, path, {}, base_address="api")

            http_client_mock.assert_requested(method, post_data=None)

            if path == self.v1_path:
                assert isinstance(resp, Customer)
            else:
                assert isinstance(resp, DeletedObject)

            assert resp.id == "abc_123"
            assert resp.object == "customer"

    @pytest.mark.anyio
    async def test_delete_methods_async(self, requestor, http_client_mock):
        for path in [self.v1_path, self.v2_path]:
            method = "delete"
            http_client_mock.stub_request(
                method,
                path=path,
                rbody=json.dumps({"id": "abc_123", "object": "customer"}),
                rcode=200,
            )

            resp = await requestor.request_async(
                method, path, {}, base_address="api"
            )

            http_client_mock.assert_requested(method, post_data=None)

            if path == self.v1_path:
                assert isinstance(resp, Customer)
            else:
                assert isinstance(resp, DeletedObject)

            assert resp.id == "abc_123"
            assert resp.object == "customer"

    def test_uses_headers(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody="{}", rcode=200
        )
        request_options: RequestOptions = {"headers": {"foo": "bar"}}
        requestor.request(
            "get",
            self.v1_path,
            {},
            options=request_options,
            base_address="api",
        )
        http_client_mock.assert_requested("get", extra_headers={"foo": "bar"})

    def test_uses_api_version(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody="{}", rcode=200
        )
        request_options: RequestOptions = {"stripe_version": "fooversion"}
        requestor.request(
            "get",
            self.v1_path,
            options=request_options,
            base_address="api",
        )
        http_client_mock.assert_requested(
            "get",
            stripe_version="fooversion",
        )

    def test_prefers_headers_api_version(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody="{}", rcode=200
        )
        request_options: RequestOptions = {
            "stripe_version": "fooversion",
            "headers": {"Stripe-Version": "barversion"},
        }
        requestor.request(
            "get",
            self.v1_path,
            {},
            options=request_options,
            base_address="api",
        )
        http_client_mock.assert_requested(
            "get",
            stripe_version="barversion",
        )

    def test_uses_instance_key(self, requestor, http_client_mock):
        key = "fookey"
        requestor = requestor._new_requestor_with_options(
            RequestOptions(api_key=key)
        )

        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody="{}", rcode=200
        )

        requestor.request("get", self.v1_path, {}, base_address="api")

        http_client_mock.assert_requested("get", api_key=key)
        assert requestor.api_key == key

    def test_uses_instance_account(self, requestor, http_client_mock):
        account = "acct_foo"
        requestor = requestor._new_requestor_with_options(
            RequestOptions(stripe_account=account)
        )

        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody="{}", rcode=200
        )

        requestor.request("get", self.v1_path, {}, base_address="api")

        http_client_mock.assert_requested(
            "get",
            stripe_account=account,
        )

    def test_removes_None_account(
        self, requestor, http_client_mock: HTTPClientMock
    ):
        """
        important test!

        If there's no context on a retrieved event, it's important that passing `stripe-account: None`
        in the generated fetch_related_object doesn't actually send the null header
        """
        account = None
        requestor = requestor._new_requestor_with_options(
            RequestOptions(stripe_account=account)
        )

        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody="{}", rcode=200
        )

        requestor.request("get", self.v1_path, {}, base_address="api")

        assert len(http_client_mock.get_all_calls()) == 1
        call = http_client_mock.get_last_call()
        assert call.headers is not None

        assert "Stripe-Account" not in call.headers

    def test_uses_instance_context(self, http_client_mock):
        context = "acct_bar"

        requestor = _APIRequestor(
            options=RequestorOptions(
                **{
                    **_GlobalRequestorOptions().to_dict(),
                    "stripe_context": context,
                }
            ),
            client=http_client_mock.get_mock_http_client(),
        )

        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody="{}", rcode=200
        )

        requestor.request("get", self.v1_path, {}, base_address="api")

        http_client_mock.assert_requested(
            "get",
            stripe_context=context,
        )

    def test_sets_default_http_client(self, mocker):
        assert not stripe.default_http_client

        _APIRequestor(client=mocker.Mock(stripe.HTTPClient))._get_http_client()

        # default_http_client is not populated if a client is provided
        assert not stripe.default_http_client

        _APIRequestor()._get_http_client()

        # default_http_client is set when no client is specified
        assert stripe.default_http_client

        new_default_client = stripe.default_http_client
        _APIRequestor()

        # the newly created client is reused
        assert stripe.default_http_client == new_default_client

    def test_uses_app_info(self, requestor, http_client_mock):
        try:
            old = stripe.app_info
            stripe.set_app_info(
                "MyAwesomePlugin",
                url="https://myawesomeplugin.info",
                version="1.2.34",
                partner_id="partner_12345",
            )

            http_client_mock.stub_request(
                "get", path=self.v1_path, rbody="{}", rcode=200
            )
            requestor.request("get", self.v1_path, {}, base_address="api")

            ua = "Stripe/v1 PythonBindings/%s" % (stripe.VERSION,)
            ua += " MyAwesomePlugin/1.2.34 (https://myawesomeplugin.info)"
            expected_app_info = {
                "name": "MyAwesomePlugin",
                "url": "https://myawesomeplugin.info",
                "version": "1.2.34",
                "partner_id": "partner_12345",
            }

            last_call = http_client_mock.get_last_call()
            last_call.assert_method("get")
            last_call.assert_header("User-Agent", ua)
            assert (
                json.loads(
                    last_call.get_raw_header("X-Stripe-Client-User-Agent")
                )["application"]
                == expected_app_info
            )
        finally:
            stripe.app_info = old

    def test_handles_failed_platform_call(
        self, requestor, mocker, http_client_mock
    ):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody="{}", rcode=200
        )

        def fail():
            raise RuntimeError

        mocker.patch("platform.platform", side_effect=fail)

        requestor.request("get", self.v1_path, {}, {}, base_address="api")

        last_call = http_client_mock.get_last_call()
        last_call.assert_method("get")
        assert (
            json.loads(last_call.get_raw_header("X-Stripe-Client-User-Agent"))[
                "platform"
            ]
            == "(disabled)"
        )

    def test_uses_given_idempotency_key(self, requestor, http_client_mock):
        method = "post"
        http_client_mock.stub_request(
            method, path=self.v1_path, rbody="{}", rcode=200
        )
        request_options: RequestOptions = {"idempotency_key": "123abc"}
        requestor.request(
            method,
            self.v1_path,
            {},
            options=request_options,
            base_address="api",
        )

        http_client_mock.assert_requested(
            method, idempotency_key="123abc", post_data=""
        )

    def test_uuid4_idempotency_key_when_not_given(
        self, requestor, http_client_mock
    ):
        method = "post"
        http_client_mock.stub_request(
            method, path=self.v1_path, rbody="{}", rcode=200
        )
        requestor.request(method, self.v1_path, {}, base_address="api")

        http_client_mock.assert_requested(
            method, idempotency_key=AnyUUID4Matcher(), post_data=""
        )

    def test_generates_default_idempotency_key_for_v2_delete(
        self, requestor, http_client_mock
    ):
        method = "delete"
        http_client_mock.stub_request(
            method, path=self.v2_path, rbody="{}", rcode=200
        )
        requestor.request(method, self.v2_path, {}, base_address="api")

        http_client_mock.assert_requested(
            method, idempotency_key=AnyUUID4Matcher()
        )

    def test_skips_generates_default_idempotency_key_for_v1_delete(
        self, requestor, http_client_mock
    ):
        method = "delete"
        http_client_mock.stub_request(
            method, path=self.v1_path, rbody="{}", rcode=200
        )
        requestor.request(method, self.v1_path, {}, base_address="api")

        http_client_mock.assert_requested(
            method, idempotency_key=IsNoneMatcher()
        )

    def test_fails_without_api_key(self, requestor):
        stripe.api_key = None

        with pytest.raises(stripe.AuthenticationError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_invalid_request_error_404(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody='{"error": {}}', rcode=404
        )

        with pytest.raises(stripe.InvalidRequestError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_invalid_request_error_400(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody='{"error": {}}', rcode=400
        )

        with pytest.raises(stripe.InvalidRequestError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_idempotency_error(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get",
            path=self.v1_path,
            rbody='{"error": {"type": "idempotency_error"}}',
            rcode=400,
        )

        with pytest.raises(stripe.IdempotencyError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_authentication_error(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody='{"error": {}}', rcode=401
        )

        with pytest.raises(stripe.AuthenticationError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_permissions_error(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody='{"error": {}}', rcode=403
        )

        with pytest.raises(stripe.PermissionError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_card_error(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get",
            path=self.v1_path,
            rbody='{"error": {"code": "invalid_expiry_year"}}',
            rcode=402,
        )

        with pytest.raises(stripe.CardError) as excinfo:
            requestor.request("get", self.v1_path, {}, base_address="api")
        assert excinfo.value.code == "invalid_expiry_year"

    def test_rate_limit_error(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody='{"error": {}}', rcode=429
        )

        with pytest.raises(stripe.RateLimitError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_old_rate_limit_error(self, requestor, http_client_mock):
        """
        Tests legacy rate limit error pre-2015-09-18
        """
        http_client_mock.stub_request(
            "get",
            path=self.v1_path,
            rbody='{"error": {"code":"rate_limit"}}',
            rcode=400,
        )

        with pytest.raises(stripe.RateLimitError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_server_error(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody='{"error": {}}', rcode=500
        )

        with pytest.raises(stripe.APIError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_invalid_json(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get", path=self.v1_path, rbody="{", rcode=200
        )

        with pytest.raises(stripe.APIError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_invalid_method(self, requestor):
        with pytest.raises(stripe.APIConnectionError):
            requestor.request("foo", "bar", base_address="api")

    def test_oauth_invalid_requestor_error(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get",
            path=self.v1_path,
            rbody='{"error": "invalid_request"}',
            rcode=400,
        )

        with pytest.raises(stripe.oauth_error.InvalidRequestError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_invalid_client_error(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get",
            path=self.v1_path,
            rbody='{"error": "invalid_client"}',
            rcode=401,
        )

        with pytest.raises(stripe.oauth_error.InvalidClientError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_invalid_grant_error(self, requestor, http_client_mock):
        http_client_mock.stub_request(
            "get",
            path=self.v1_path,
            rbody='{"error": "invalid_grant"}',
            rcode=400,
        )

        with pytest.raises(stripe.oauth_error.InvalidGrantError):
            requestor.request("get", self.v1_path, {}, base_address="api")

    def test_extract_error_from_stream_request_for_bytes(
        self, requestor, http_client_mock
    ):
        http_client_mock.stub_request(
            "get",
            path=self.v1_path,
            rbody=io.BytesIO(b'{"error": "invalid_grant"}'),
            rcode=400,
        )

        with pytest.raises(stripe.oauth_error.InvalidGrantError):
            requestor.request_stream(
                "get", self.v1_path, {}, base_address="api"
            )

    def test_extract_error_from_stream_request_for_response(
        self, requestor, http_client_mock
    ):
        # Responses don't have getvalue, they only have a read method.
        http_client_mock.stub_request(
            "get",
            path=self.v1_path,
            rbody=urllib3.response.HTTPResponse(
                body=io.BytesIO(b'{"error": "invalid_grant"}'),
                preload_content=False,
            ),
            rcode=400,
        )

        with pytest.raises(stripe.oauth_error.InvalidGrantError):
            requestor.request_stream(
                "get", self.v1_path, {}, base_address="api"
            )

    def test_raw_request_with_file_param(self, requestor, http_client_mock):
        test_file = tempfile.NamedTemporaryFile()
        test_file.write("\u263a".encode("utf-16"))
        test_file.seek(0)
        method = "post"
        path = "/v1/files"
        params = {"file": test_file, "purpose": "dispute_evidence"}
        supplied_headers = {"Content-Type": "multipart/form-data"}
        http_client_mock.stub_request(method, path=path, rbody="{}", rcode=200)
        requestor.request(
            method,
            path,
            params,
            supplied_headers,
            base_address="api",
        )
        assert supplied_headers["Content-Type"] == "multipart/form-data"


class TestDefaultClient(object):
    @pytest.fixture(autouse=True)
    def setup_stripe(self):
        orig_attrs = {
            "api_key": stripe.api_key,
            "default_http_client": stripe.default_http_client,
        }
        stripe.api_key = "sk_test_123"
        yield
        stripe.api_key = orig_attrs["api_key"]
        stripe.default_http_client = orig_attrs["default_http_client"]

    def test_default_http_client_called(self, http_client_mock):
        http_client_mock.stub_request(
            "get",
            path="/v1/charges",
            query_string="limit=3",
            rbody='{"object": "list", "data": []}',
            rcode=200,
            rheaders={},
        )

        stripe.Charge.list(limit=3)

        last_call = http_client_mock.get_last_call()

        last_call.assert_method("get")
        last_call.assert_abs_url("https://api.stripe.com/v1/charges?limit=3")
        last_call.assert_post_data(None)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.issuing._card import Card
    from stripe.params.issuing._card_create_params import CardCreateParams
    from stripe.params.issuing._card_list_params import CardListParams
    from stripe.params.issuing._card_retrieve_params import CardRetrieveParams
    from stripe.params.issuing._card_update_params import CardUpdateParams


class CardService(StripeService):
    def list(
        self,
        params: Optional["CardListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Card]":
        """
        Returns a list of Issuing Card objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        return cast(
            "ListObject[Card]",
            self._request(
                "get",
                "/v1/issuing/cards",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["CardListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Card]":
        """
        Returns a list of Issuing Card objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        return cast(
            "ListObject[Card]",
            await self._request_async(
                "get",
                "/v1/issuing/cards",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "CardCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Card":
        """
        Creates an Issuing Card object.
        """
        return cast(
            "Card",
            self._request(
                "post",
                "/v1/issuing/cards",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "CardCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Card":
        """
        Creates an Issuing Card object.
        """
        return cast(
            "Card",
            await self._request_async(
                "post",
                "/v1/issuing/cards",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        card: str,
        params: Optional["CardRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Card":
        """
        Retrieves an Issuing Card object.
        """
        return cast(
            "Card",
            self._request(
                "get",
                "/v1/issuing/cards/{card}".format(card=sanitize_id(card)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        card: str,
        params: Optional["CardRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Card":
        """
        Retrieves an Issuing Card object.
        """
        return cast(
            "Card",
            await self._request_async(
                "get",
                "/v1/issuing/cards/{card}".format(card=sanitize_id(card)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        card: str,
        params: Optional["CardUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Card":
        """
        Updates the specified Issuing Card object by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        return cast(
            "Card",
            self._request(
                "post",
                "/v1/issuing/cards/{card}".format(card=sanitize_id(card)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        card: str,
        params: Optional["CardUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Card":
        """
        Updates the specified Issuing Card object by setting the values of the parameters passed. Any parameters not provided will be left unchanged.
        """
        return cast(
            "Card",
            await self._request_async(
                "post",
                "/v1/issuing/cards/{card}".format(card=sanitize_id(card)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._product_feature import ProductFeature
    from stripe._request_options import RequestOptions
    from stripe.params._product_feature_create_params import (
        ProductFeatureCreateParams,
    )
    from stripe.params._product_feature_delete_params import (
        ProductFeatureDeleteParams,
    )
    from stripe.params._product_feature_list_params import (
        ProductFeatureListParams,
    )
    from stripe.params._product_feature_retrieve_params import (
        ProductFeatureRetrieveParams,
    )


class ProductFeatureService(StripeService):
    def delete(
        self,
        product: str,
        id: str,
        params: Optional["ProductFeatureDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ProductFeature":
        """
        Deletes the feature attachment to a product
        """
        return cast(
            "ProductFeature",
            self._request(
                "delete",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def delete_async(
        self,
        product: str,
        id: str,
        params: Optional["ProductFeatureDeleteParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ProductFeature":
        """
        Deletes the feature attachment to a product
        """
        return cast(
            "ProductFeature",
            await self._request_async(
                "delete",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        product: str,
        id: str,
        params: Optional["ProductFeatureRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ProductFeature":
        """
        Retrieves a product_feature, which represents a feature attachment to a product
        """
        return cast(
            "ProductFeature",
            self._request(
                "get",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        product: str,
        id: str,
        params: Optional["ProductFeatureRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ProductFeature":
        """
        Retrieves a product_feature, which represents a feature attachment to a product
        """
        return cast(
            "ProductFeature",
            await self._request_async(
                "get",
                "/v1/products/{product}/features/{id}".format(
                    product=sanitize_id(product),
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def list(
        self,
        product: str,
        params: Optional["ProductFeatureListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ProductFeature]":
        """
        Retrieve a list of features for a product
        """
        return cast(
            "ListObject[ProductFeature]",
            self._request(
                "get",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        product: str,
        params: Optional["ProductFeatureListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[ProductFeature]":
        """
        Retrieve a list of features for a product
        """
        return cast(
            "ListObject[ProductFeature]",
            await self._request_async(
                "get",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        product: str,
        params: "ProductFeatureCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ProductFeature":
        """
        Creates a product_feature, which represents a feature attachment to a product
        """
        return cast(
            "ProductFeature",
            self._request(
                "post",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        product: str,
        params: "ProductFeatureCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "ProductFeature":
        """
        Creates a product_feature, which represents a feature attachment to a product
        """
        return cast(
            "ProductFeature",
            await self._request_async(
                "post",
                "/v1/products/{product}/features".format(
                    product=sanitize_id(product),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import NotRequired, TypedDict


class CustomerPaymentSourceVerifyParams(TypedDict):
    amounts: NotRequired[List[int]]
    """
    Two positive integers, in *cents*, equal to the values of the microdeposits sent to the bank account.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List
from typing_extensions import Literal, NotRequired, TypedDict


class TokenCreateParams(RequestOptions):
    account: NotRequired["TokenCreateParamsAccount"]
    """
    Information for the account this token represents.
    """
    bank_account: NotRequired["TokenCreateParamsBankAccount"]
    """
    The bank account this token will represent.
    """
    card: NotRequired["TokenCreateParamsCard|str"]
    """
    The card this token will represent. If you also pass in a customer, the card must be the ID of a card belonging to the customer. Otherwise, if you do not pass in a customer, this is a dictionary containing a user's credit card details, with the options described below.
    """
    customer: NotRequired[str]
    """
    Create a token for the customer, which is owned by the application's account. You can only use this with an [OAuth access token](https://stripe.com/docs/connect/standard-accounts) or [Stripe-Account header](https://stripe.com/docs/connect/authentication). Learn more about [cloning saved payment methods](https://stripe.com/docs/connect/cloning-saved-payment-methods).
    """
    cvc_update: NotRequired["TokenCreateParamsCvcUpdate"]
    """
    The updated CVC value this token represents.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    person: NotRequired["TokenCreateParamsPerson"]
    """
    Information for the person this token represents.
    """
    pii: NotRequired["TokenCreateParamsPii"]
    """
    The PII this token represents.
    """


class TokenCreateParamsAccount(TypedDict):
    business_type: NotRequired[
        Literal["company", "government_entity", "individual", "non_profit"]
    ]
    """
    The business type.
    """
    company: NotRequired["TokenCreateParamsAccountCompany"]
    """
    Information about the company or business.
    """
    individual: NotRequired["TokenCreateParamsAccountIndividual"]
    """
    Information about the person represented by the account.
    """
    tos_shown_and_accepted: NotRequired[bool]
    """
    Whether the user described by the data in the token has been shown [the Stripe Connected Account Agreement](https://docs.stripe.com/connect/account-tokens#stripe-connected-account-agreement). When creating an account token to create a new Connect account, this value must be `true`.
    """


class TokenCreateParamsAccountCompany(TypedDict):
    address: NotRequired["TokenCreateParamsAccountCompanyAddress"]
    """
    The company's primary address.
    """
    address_kana: NotRequired["TokenCreateParamsAccountCompanyAddressKana"]
    """
    The Kana variation of the company's primary address (Japan only).
    """
    address_kanji: NotRequired["TokenCreateParamsAccountCompanyAddressKanji"]
    """
    The Kanji variation of the company's primary address (Japan only).
    """
    directors_provided: NotRequired[bool]
    """
    Whether the company's directors have been provided. Set this Boolean to `true` after creating all the company's directors with [the Persons API](https://docs.stripe.com/api/persons) for accounts with a `relationship.director` requirement. This value is not automatically set to `true` after creating directors, so it needs to be updated to indicate all directors have been provided.
    """
    directorship_declaration: NotRequired[
        "TokenCreateParamsAccountCompanyDirectorshipDeclaration"
    ]
    """
    This hash is used to attest that the directors information provided to Stripe is both current and correct.
    """
    executives_provided: NotRequired[bool]
    """
    Whether the company's executives have been provided. Set this Boolean to `true` after creating all the company's executives with [the Persons API](https://docs.stripe.com/api/persons) for accounts with a `relationship.executive` requirement.
    """
    export_license_id: NotRequired[str]
    """
    The export license ID number of the company, also referred as Import Export Code (India only).
    """
    export_purpose_code: NotRequired[str]
    """
    The purpose code to use for export transactions (India only).
    """
    name: NotRequired[str]
    """
    The company's legal name.
    """
    name_kana: NotRequired[str]
    """
    The Kana variation of the company's legal name (Japan only).
    """
    name_kanji: NotRequired[str]
    """
    The Kanji variation of the company's legal name (Japan only).
    """
    owners_provided: NotRequired[bool]
    """
    Whether the company's owners have been provided. Set this Boolean to `true` after creating all the company's owners with [the Persons API](https://docs.stripe.com/api/persons) for accounts with a `relationship.owner` requirement.
    """
    ownership_declaration: NotRequired[
        "TokenCreateParamsAccountCompanyOwnershipDeclaration"
    ]
    """
    This hash is used to attest that the beneficial owner information provided to Stripe is both current and correct.
    """
    ownership_declaration_shown_and_signed: NotRequired[bool]
    """
    Whether the user described by the data in the token has been shown the Ownership Declaration and indicated that it is correct.
    """
    ownership_exemption_reason: NotRequired[
        "Literal['']|Literal['qualified_entity_exceeds_ownership_threshold', 'qualifies_as_financial_institution']"
    ]
    """
    This value is used to determine if a business is exempt from providing ultimate beneficial owners. See [this support article](https://support.stripe.com/questions/exemption-from-providing-ownership-details) and [changelog](https://docs.stripe.com/changelog/acacia/2025-01-27/ownership-exemption-reason-accounts-api) for more details.
    """
    phone: NotRequired[str]
    """
    The company's phone number (used for verification).
    """
    registration_date: NotRequired[
        "Literal['']|TokenCreateParamsAccountCompanyRegistrationDate"
    ]
    """
    When the business was incorporated or registered.
    """
    registration_number: NotRequired[str]
    """
    The identification number given to a company when it is registered or incorporated, if distinct from the identification number used for filing taxes. (Examples are the CIN for companies and LLP IN for partnerships in India, and the Company Registration Number in Hong Kong).
    """
    representative_declaration: NotRequired[
        "TokenCreateParamsAccountCompanyRepresentativeDeclaration"
    ]
    """
    This hash is used to attest that the representative is authorized to act as the representative of their legal entity.
    """
    structure: NotRequired[
        "Literal['']|Literal['free_zone_establishment', 'free_zone_llc', 'government_instrumentality', 'governmental_unit', 'incorporated_non_profit', 'incorporated_partnership', 'limited_liability_partnership', 'llc', 'multi_member_llc', 'private_company', 'private_corporation', 'private_partnership', 'public_company', 'public_corporation', 'public_partnership', 'registered_charity', 'single_member_llc', 'sole_establishment', 'sole_proprietorship', 'tax_exempt_government_instrumentality', 'unincorporated_association', 'unincorporated_non_profit', 'unincorporated_partnership']"
    ]
    """
    The category identifying the legal structure of the company or legal entity. See [Business structure](https://docs.stripe.com/connect/identity-verification#business-structure) for more details. Pass an empty string to unset this value.
    """
    tax_id: NotRequired[str]
    """
    The business ID number of the company, as appropriate for the company's country. (Examples are an Employer ID Number in the U.S., a Business Number in Canada, or a Company Number in the UK.)
    """
    tax_id_registrar: NotRequired[str]
    """
    The jurisdiction in which the `tax_id` is registered (Germany-based companies only).
    """
    vat_id: NotRequired[str]
    """
    The VAT number of the company.
    """
    verification: NotRequired["TokenCreateParamsAccountCompanyVerification"]
    """
    Information on the verification state of the company.
    """


class TokenCreateParamsAccountCompanyAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """


class TokenCreateParamsAccountCompanyAddressKana(TypedDict):
    city: NotRequired[str]
    """
    City or ward.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Block or building number.
    """
    line2: NotRequired[str]
    """
    Building details.
    """
    postal_code: NotRequired[str]
    """
    Postal code.
    """
    state: NotRequired[str]
    """
    Prefecture.
    """
    town: NotRequired[str]
    """
    Town or cho-me.
    """


class TokenCreateParamsAccountCompanyAddressKanji(TypedDict):
    city: NotRequired[str]
    """
    City or ward.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Block or building number.
    """
    line2: NotRequired[str]
    """
    Building details.
    """
    postal_code: NotRequired[str]
    """
    Postal code.
    """
    state: NotRequired[str]
    """
    Prefecture.
    """
    town: NotRequired[str]
    """
    Town or cho-me.
    """


class TokenCreateParamsAccountCompanyDirectorshipDeclaration(TypedDict):
    date: NotRequired[int]
    """
    The Unix timestamp marking when the directorship declaration attestation was made.
    """
    ip: NotRequired[str]
    """
    The IP address from which the directorship declaration attestation was made.
    """
    user_agent: NotRequired[str]
    """
    The user agent of the browser from which the directorship declaration attestation was made.
    """


class TokenCreateParamsAccountCompanyOwnershipDeclaration(TypedDict):
    date: NotRequired[int]
    """
    The Unix timestamp marking when the beneficial owner attestation was made.
    """
    ip: NotRequired[str]
    """
    The IP address from which the beneficial owner attestation was made.
    """
    user_agent: NotRequired[str]
    """
    The user agent of the browser from which the beneficial owner attestation was made.
    """


class TokenCreateParamsAccountCompanyRegistrationDate(TypedDict):
    day: int
    """
    The day of registration, between 1 and 31.
    """
    month: int
    """
    The month of registration, between 1 and 12.
    """
    year: int
    """
    The four-digit year of registration.
    """


class TokenCreateParamsAccountCompanyRepresentativeDeclaration(TypedDict):
    date: NotRequired[int]
    """
    The Unix timestamp marking when the representative declaration attestation was made.
    """
    ip: NotRequired[str]
    """
    The IP address from which the representative declaration attestation was made.
    """
    user_agent: NotRequired[str]
    """
    The user agent of the browser from which the representative declaration attestation was made.
    """


class TokenCreateParamsAccountCompanyVerification(TypedDict):
    document: NotRequired[
        "TokenCreateParamsAccountCompanyVerificationDocument"
    ]
    """
    A document verifying the business.
    """


class TokenCreateParamsAccountCompanyVerificationDocument(TypedDict):
    back: NotRequired[str]
    """
    The back of a document returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `additional_verification`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """
    front: NotRequired[str]
    """
    The front of a document returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `additional_verification`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """


class TokenCreateParamsAccountIndividual(TypedDict):
    address: NotRequired["TokenCreateParamsAccountIndividualAddress"]
    """
    The individual's primary address.
    """
    address_kana: NotRequired["TokenCreateParamsAccountIndividualAddressKana"]
    """
    The Kana variation of the individual's primary address (Japan only).
    """
    address_kanji: NotRequired[
        "TokenCreateParamsAccountIndividualAddressKanji"
    ]
    """
    The Kanji variation of the individual's primary address (Japan only).
    """
    dob: NotRequired["Literal['']|TokenCreateParamsAccountIndividualDob"]
    """
    The individual's date of birth.
    """
    email: NotRequired[str]
    """
    The individual's email address.
    """
    first_name: NotRequired[str]
    """
    The individual's first name.
    """
    first_name_kana: NotRequired[str]
    """
    The Kana variation of the individual's first name (Japan only).
    """
    first_name_kanji: NotRequired[str]
    """
    The Kanji variation of the individual's first name (Japan only).
    """
    full_name_aliases: NotRequired["Literal['']|List[str]"]
    """
    A list of alternate names or aliases that the individual is known by.
    """
    gender: NotRequired[str]
    """
    The individual's gender
    """
    id_number: NotRequired[str]
    """
    The government-issued ID number of the individual, as appropriate for the representative's country. (Examples are a Social Security Number in the U.S., or a Social Insurance Number in Canada). Instead of the number itself, you can also provide a [PII token created with Stripe.js](https://docs.stripe.com/js/tokens/create_token?type=pii).
    """
    id_number_secondary: NotRequired[str]
    """
    The government-issued secondary ID number of the individual, as appropriate for the representative's country, will be used for enhanced verification checks. In Thailand, this would be the laser code found on the back of an ID card. Instead of the number itself, you can also provide a [PII token created with Stripe.js](https://docs.stripe.com/js/tokens/create_token?type=pii).
    """
    last_name: NotRequired[str]
    """
    The individual's last name.
    """
    last_name_kana: NotRequired[str]
    """
    The Kana variation of the individual's last name (Japan only).
    """
    last_name_kanji: NotRequired[str]
    """
    The Kanji variation of the individual's last name (Japan only).
    """
    maiden_name: NotRequired[str]
    """
    The individual's maiden name.
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    phone: NotRequired[str]
    """
    The individual's phone number.
    """
    political_exposure: NotRequired[Literal["existing", "none"]]
    """
    Indicates if the person or any of their representatives, family members, or other closely related persons, declares that they hold or have held an important public job or function, in any jurisdiction.
    """
    registered_address: NotRequired[
        "TokenCreateParamsAccountIndividualRegisteredAddress"
    ]
    """
    The individual's registered address.
    """
    relationship: NotRequired["TokenCreateParamsAccountIndividualRelationship"]
    """
    Describes the person's relationship to the account.
    """
    ssn_last_4: NotRequired[str]
    """
    The last four digits of the individual's Social Security Number (U.S. only).
    """
    verification: NotRequired["TokenCreateParamsAccountIndividualVerification"]
    """
    The individual's verification document information.
    """


class TokenCreateParamsAccountIndividualAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """


class TokenCreateParamsAccountIndividualAddressKana(TypedDict):
    city: NotRequired[str]
    """
    City or ward.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Block or building number.
    """
    line2: NotRequired[str]
    """
    Building details.
    """
    postal_code: NotRequired[str]
    """
    Postal code.
    """
    state: NotRequired[str]
    """
    Prefecture.
    """
    town: NotRequired[str]
    """
    Town or cho-me.
    """


class TokenCreateParamsAccountIndividualAddressKanji(TypedDict):
    city: NotRequired[str]
    """
    City or ward.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Block or building number.
    """
    line2: NotRequired[str]
    """
    Building details.
    """
    postal_code: NotRequired[str]
    """
    Postal code.
    """
    state: NotRequired[str]
    """
    Prefecture.
    """
    town: NotRequired[str]
    """
    Town or cho-me.
    """


class TokenCreateParamsAccountIndividualDob(TypedDict):
    day: int
    """
    The day of birth, between 1 and 31.
    """
    month: int
    """
    The month of birth, between 1 and 12.
    """
    year: int
    """
    The four-digit year of birth.
    """


class TokenCreateParamsAccountIndividualRegisteredAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """


class TokenCreateParamsAccountIndividualRelationship(TypedDict):
    director: NotRequired[bool]
    """
    Whether the person is a director of the account's legal entity. Directors are typically members of the governing board of the company, or responsible for ensuring the company meets its regulatory obligations.
    """
    executive: NotRequired[bool]
    """
    Whether the person has significant responsibility to control, manage, or direct the organization.
    """
    owner: NotRequired[bool]
    """
    Whether the person is an owner of the account's legal entity.
    """
    percent_ownership: NotRequired["Literal['']|float"]
    """
    The percent owned by the person of the account's legal entity.
    """
    title: NotRequired[str]
    """
    The person's title (e.g., CEO, Support Engineer).
    """


class TokenCreateParamsAccountIndividualVerification(TypedDict):
    additional_document: NotRequired[
        "TokenCreateParamsAccountIndividualVerificationAdditionalDocument"
    ]
    """
    A document showing address, either a passport, local ID card, or utility bill from a well-known utility company.
    """
    document: NotRequired[
        "TokenCreateParamsAccountIndividualVerificationDocument"
    ]
    """
    An identifying document, either a passport or local ID card.
    """


class TokenCreateParamsAccountIndividualVerificationAdditionalDocument(
    TypedDict,
):
    back: NotRequired[str]
    """
    The back of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """
    front: NotRequired[str]
    """
    The front of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """


class TokenCreateParamsAccountIndividualVerificationDocument(TypedDict):
    back: NotRequired[str]
    """
    The back of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """
    front: NotRequired[str]
    """
    The front of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """


class TokenCreateParamsBankAccount(TypedDict):
    account_holder_name: NotRequired[str]
    """
    The name of the person or business that owns the bank account. This field is required when attaching the bank account to a `Customer` object.
    """
    account_holder_type: NotRequired[Literal["company", "individual"]]
    """
    The type of entity that holds the account. It can be `company` or `individual`. This field is required when attaching the bank account to a `Customer` object.
    """
    account_number: str
    """
    The account number for the bank account, in string form. Must be a checking account.
    """
    account_type: NotRequired[Literal["checking", "futsu", "savings", "toza"]]
    """
    The bank account type. This can only be `checking` or `savings` in most countries. In Japan, this can only be `futsu` or `toza`.
    """
    country: str
    """
    The country in which the bank account is located.
    """
    currency: NotRequired[str]
    """
    The currency the bank account is in. This must be a country/currency pairing that [Stripe supports.](https://stripe.com/docs/payouts)
    """
    payment_method: NotRequired[str]
    """
    The ID of a Payment Method with a `type` of `us_bank_account`. The Payment Method's bank account information will be copied and returned as a Bank Account Token. This parameter is exclusive with respect to all other parameters in the `bank_account` hash. You must include the top-level `customer` parameter if the Payment Method is attached to a `Customer` object. If the Payment Method is not attached to a `Customer` object, it will be consumed and cannot be used again. You may not use Payment Methods which were created by a Setup Intent with `attach_to_self=true`.
    """
    routing_number: NotRequired[str]
    """
    The routing number, sort code, or other country-appropriate institution number for the bank account. For US bank accounts, this is required and should be the ACH routing number, not the wire routing number. If you are providing an IBAN for `account_number`, this field is not required.
    """


class TokenCreateParamsCard(TypedDict):
    address_city: NotRequired[str]
    """
    City / District / Suburb / Town / Village.
    """
    address_country: NotRequired[str]
    """
    Billing address country, if provided.
    """
    address_line1: NotRequired[str]
    """
    Address line 1 (Street address / PO Box / Company name).
    """
    address_line2: NotRequired[str]
    """
    Address line 2 (Apartment / Suite / Unit / Building).
    """
    address_state: NotRequired[str]
    """
    State / County / Province / Region.
    """
    address_zip: NotRequired[str]
    """
    ZIP or postal code.
    """
    currency: NotRequired[str]
    """
    Required in order to add the card to an account; in all other cases, this parameter is not used. When added to an account, the card (which must be a debit card) can be used as a transfer destination for funds in this currency.
    """
    cvc: NotRequired[str]
    """
    Card security code. Highly recommended to always include this value.
    """
    exp_month: str
    """
    Two-digit number representing the card's expiration month.
    """
    exp_year: str
    """
    Two- or four-digit number representing the card's expiration year.
    """
    name: NotRequired[str]
    """
    Cardholder's full name.
    """
    networks: NotRequired["TokenCreateParamsCardNetworks"]
    """
    Contains information about card networks used to process the payment.
    """
    number: str
    """
    The card number, as a string without any separators.
    """


class TokenCreateParamsCardNetworks(TypedDict):
    preferred: NotRequired[Literal["cartes_bancaires", "mastercard", "visa"]]
    """
    The customer's preferred card network for co-branded cards. Supports `cartes_bancaires`, `mastercard`, or `visa`. Selection of a network that does not apply to the card will be stored as `invalid_preference` on the card.
    """


class TokenCreateParamsCvcUpdate(TypedDict):
    cvc: str
    """
    The CVC value, in string form.
    """


class TokenCreateParamsPerson(TypedDict):
    additional_tos_acceptances: NotRequired[
        "TokenCreateParamsPersonAdditionalTosAcceptances"
    ]
    """
    Details on the legal guardian's or authorizer's acceptance of the required Stripe agreements.
    """
    address: NotRequired["TokenCreateParamsPersonAddress"]
    """
    The person's address.
    """
    address_kana: NotRequired["TokenCreateParamsPersonAddressKana"]
    """
    The Kana variation of the person's address (Japan only).
    """
    address_kanji: NotRequired["TokenCreateParamsPersonAddressKanji"]
    """
    The Kanji variation of the person's address (Japan only).
    """
    dob: NotRequired["Literal['']|TokenCreateParamsPersonDob"]
    """
    The person's date of birth.
    """
    documents: NotRequired["TokenCreateParamsPersonDocuments"]
    """
    Documents that may be submitted to satisfy various informational requests.
    """
    email: NotRequired[str]
    """
    The person's email address.
    """
    first_name: NotRequired[str]
    """
    The person's first name.
    """
    first_name_kana: NotRequired[str]
    """
    The Kana variation of the person's first name (Japan only).
    """
    first_name_kanji: NotRequired[str]
    """
    The Kanji variation of the person's first name (Japan only).
    """
    full_name_aliases: NotRequired["Literal['']|List[str]"]
    """
    A list of alternate names or aliases that the person is known by.
    """
    gender: NotRequired[str]
    """
    The person's gender (International regulations require either "male" or "female").
    """
    id_number: NotRequired[str]
    """
    The person's ID number, as appropriate for their country. For example, a social security number in the U.S., social insurance number in Canada, etc. Instead of the number itself, you can also provide a [PII token provided by Stripe.js](https://docs.stripe.com/js/tokens/create_token?type=pii).
    """
    id_number_secondary: NotRequired[str]
    """
    The person's secondary ID number, as appropriate for their country, will be used for enhanced verification checks. In Thailand, this would be the laser code found on the back of an ID card. Instead of the number itself, you can also provide a [PII token provided by Stripe.js](https://docs.stripe.com/js/tokens/create_token?type=pii).
    """
    last_name: NotRequired[str]
    """
    The person's last name.
    """
    last_name_kana: NotRequired[str]
    """
    The Kana variation of the person's last name (Japan only).
    """
    last_name_kanji: NotRequired[str]
    """
    The Kanji variation of the person's last name (Japan only).
    """
    maiden_name: NotRequired[str]
    """
    The person's maiden name.
    """
    metadata: NotRequired["Literal['']|Dict[str, str]"]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    nationality: NotRequired[str]
    """
    The country where the person is a national. Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)), or "XX" if unavailable.
    """
    phone: NotRequired[str]
    """
    The person's phone number.
    """
    political_exposure: NotRequired[Literal["existing", "none"]]
    """
    Indicates if the person or any of their representatives, family members, or other closely related persons, declares that they hold or have held an important public job or function, in any jurisdiction.
    """
    registered_address: NotRequired["TokenCreateParamsPersonRegisteredAddress"]
    """
    The person's registered address.
    """
    relationship: NotRequired["TokenCreateParamsPersonRelationship"]
    """
    The relationship that this person has with the account's legal entity.
    """
    ssn_last_4: NotRequired[str]
    """
    The last four digits of the person's Social Security number (U.S. only).
    """
    us_cfpb_data: NotRequired["TokenCreateParamsPersonUsCfpbData"]
    """
    Demographic data related to the person.
    """
    verification: NotRequired["TokenCreateParamsPersonVerification"]
    """
    The person's verification status.
    """


class TokenCreateParamsPersonAdditionalTosAcceptances(TypedDict):
    account: NotRequired[
        "TokenCreateParamsPersonAdditionalTosAcceptancesAccount"
    ]
    """
    Details on the legal guardian's acceptance of the main Stripe service agreement.
    """


class TokenCreateParamsPersonAdditionalTosAcceptancesAccount(TypedDict):
    date: NotRequired[int]
    """
    The Unix timestamp marking when the account representative accepted the service agreement.
    """
    ip: NotRequired[str]
    """
    The IP address from which the account representative accepted the service agreement.
    """
    user_agent: NotRequired["Literal['']|str"]
    """
    The user agent of the browser from which the account representative accepted the service agreement.
    """


class TokenCreateParamsPersonAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """


class TokenCreateParamsPersonAddressKana(TypedDict):
    city: NotRequired[str]
    """
    City or ward.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Block or building number.
    """
    line2: NotRequired[str]
    """
    Building details.
    """
    postal_code: NotRequired[str]
    """
    Postal code.
    """
    state: NotRequired[str]
    """
    Prefecture.
    """
    town: NotRequired[str]
    """
    Town or cho-me.
    """


class TokenCreateParamsPersonAddressKanji(TypedDict):
    city: NotRequired[str]
    """
    City or ward.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Block or building number.
    """
    line2: NotRequired[str]
    """
    Building details.
    """
    postal_code: NotRequired[str]
    """
    Postal code.
    """
    state: NotRequired[str]
    """
    Prefecture.
    """
    town: NotRequired[str]
    """
    Town or cho-me.
    """


class TokenCreateParamsPersonDob(TypedDict):
    day: int
    """
    The day of birth, between 1 and 31.
    """
    month: int
    """
    The month of birth, between 1 and 12.
    """
    year: int
    """
    The four-digit year of birth.
    """


class TokenCreateParamsPersonDocuments(TypedDict):
    company_authorization: NotRequired[
        "TokenCreateParamsPersonDocumentsCompanyAuthorization"
    ]
    """
    One or more documents that demonstrate proof that this person is authorized to represent the company.
    """
    passport: NotRequired["TokenCreateParamsPersonDocumentsPassport"]
    """
    One or more documents showing the person's passport page with photo and personal data.
    """
    visa: NotRequired["TokenCreateParamsPersonDocumentsVisa"]
    """
    One or more documents showing the person's visa required for living in the country where they are residing.
    """


class TokenCreateParamsPersonDocumentsCompanyAuthorization(TypedDict):
    files: NotRequired[List[str]]
    """
    One or more document ids returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `account_requirement`.
    """


class TokenCreateParamsPersonDocumentsPassport(TypedDict):
    files: NotRequired[List[str]]
    """
    One or more document ids returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `account_requirement`.
    """


class TokenCreateParamsPersonDocumentsVisa(TypedDict):
    files: NotRequired[List[str]]
    """
    One or more document ids returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `account_requirement`.
    """


class TokenCreateParamsPersonRegisteredAddress(TypedDict):
    city: NotRequired[str]
    """
    City, district, suburb, town, or village.
    """
    country: NotRequired[str]
    """
    Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
    """
    line1: NotRequired[str]
    """
    Address line 1, such as the street, PO Box, or company name.
    """
    line2: NotRequired[str]
    """
    Address line 2, such as the apartment, suite, unit, or building.
    """
    postal_code: NotRequired[str]
    """
    ZIP or postal code.
    """
    state: NotRequired[str]
    """
    State, county, province, or region.
    """


class TokenCreateParamsPersonRelationship(TypedDict):
    authorizer: NotRequired[bool]
    """
    Whether the person is the authorizer of the account's representative.
    """
    director: NotRequired[bool]
    """
    Whether the person is a director of the account's legal entity. Directors are typically members of the governing board of the company, or responsible for ensuring the company meets its regulatory obligations.
    """
    executive: NotRequired[bool]
    """
    Whether the person has significant responsibility to control, manage, or direct the organization.
    """
    legal_guardian: NotRequired[bool]
    """
    Whether the person is the legal guardian of the account's representative.
    """
    owner: NotRequired[bool]
    """
    Whether the person is an owner of the account's legal entity.
    """
    percent_ownership: NotRequired["Literal['']|float"]
    """
    The percent owned by the person of the account's legal entity.
    """
    representative: NotRequired[bool]
    """
    Whether the person is authorized as the primary representative of the account. This is the person nominated by the business to provide information about themselves, and general information about the account. There can only be one representative at any given time. At the time the account is created, this person should be set to the person responsible for opening the account.
    """
    title: NotRequired[str]
    """
    The person's title (e.g., CEO, Support Engineer).
    """


class TokenCreateParamsPersonUsCfpbData(TypedDict):
    ethnicity_details: NotRequired[
        "TokenCreateParamsPersonUsCfpbDataEthnicityDetails"
    ]
    """
    The persons ethnicity details
    """
    race_details: NotRequired["TokenCreateParamsPersonUsCfpbDataRaceDetails"]
    """
    The persons race details
    """
    self_identified_gender: NotRequired[str]
    """
    The persons self-identified gender
    """


class TokenCreateParamsPersonUsCfpbDataEthnicityDetails(TypedDict):
    ethnicity: NotRequired[
        List[
            Literal[
                "cuban",
                "hispanic_or_latino",
                "mexican",
                "not_hispanic_or_latino",
                "other_hispanic_or_latino",
                "prefer_not_to_answer",
                "puerto_rican",
            ]
        ]
    ]
    """
    The persons ethnicity
    """
    ethnicity_other: NotRequired[str]
    """
    Please specify your origin, when other is selected.
    """


class TokenCreateParamsPersonUsCfpbDataRaceDetails(TypedDict):
    race: NotRequired[
        List[
            Literal[
                "african_american",
                "american_indian_or_alaska_native",
                "asian",
                "asian_indian",
                "black_or_african_american",
                "chinese",
                "ethiopian",
                "filipino",
                "guamanian_or_chamorro",
                "haitian",
                "jamaican",
                "japanese",
                "korean",
                "native_hawaiian",
                "native_hawaiian_or_other_pacific_islander",
                "nigerian",
                "other_asian",
                "other_black_or_african_american",
                "other_pacific_islander",
                "prefer_not_to_answer",
                "samoan",
                "somali",
                "vietnamese",
                "white",
            ]
        ]
    ]
    """
    The persons race.
    """
    race_other: NotRequired[str]
    """
    Please specify your race, when other is selected.
    """


class TokenCreateParamsPersonVerification(TypedDict):
    additional_document: NotRequired[
        "TokenCreateParamsPersonVerificationAdditionalDocument"
    ]
    """
    A document showing address, either a passport, local ID card, or utility bill from a well-known utility company.
    """
    document: NotRequired["TokenCreateParamsPersonVerificationDocument"]
    """
    An identifying document, either a passport or local ID card.
    """


class TokenCreateParamsPersonVerificationAdditionalDocument(TypedDict):
    back: NotRequired[str]
    """
    The back of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """
    front: NotRequired[str]
    """
    The front of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """


class TokenCreateParamsPersonVerificationDocument(TypedDict):
    back: NotRequired[str]
    """
    The back of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """
    front: NotRequired[str]
    """
    The front of an ID returned by a [file upload](https://stripe.com/docs/api#create_file) with a `purpose` value of `identity_document`. The uploaded file needs to be a color image (smaller than 8,000px by 8,000px), in JPG, PNG, or PDF format, and less than 10 MB in size.
    """


class TokenCreateParamsPii(TypedDict):
    id_number: NotRequired[str]
    """
    The `id_number` for the PII, in string form.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._deletable_api_resource import DeletableAPIResource
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Dict, List, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._webhook_endpoint_create_params import (
        WebhookEndpointCreateParams,
    )
    from stripe.params._webhook_endpoint_delete_params import (
        WebhookEndpointDeleteParams,
    )
    from stripe.params._webhook_endpoint_list_params import (
        WebhookEndpointListParams,
    )
    from stripe.params._webhook_endpoint_modify_params import (
        WebhookEndpointModifyParams,
    )
    from stripe.params._webhook_endpoint_retrieve_params import (
        WebhookEndpointRetrieveParams,
    )


class WebhookEndpoint(
    CreateableAPIResource["WebhookEndpoint"],
    DeletableAPIResource["WebhookEndpoint"],
    ListableAPIResource["WebhookEndpoint"],
    UpdateableAPIResource["WebhookEndpoint"],
):
    """
    You can configure [webhook endpoints](https://docs.stripe.com/webhooks/) via the API to be
    notified about events that happen in your Stripe account or connected
    accounts.

    Most users configure webhooks from [the dashboard](https://dashboard.stripe.com/webhooks), which provides a user interface for registering and testing your webhook endpoints.

    Related guide: [Setting up webhooks](https://docs.stripe.com/webhooks/configure)
    """

    OBJECT_NAME: ClassVar[Literal["webhook_endpoint"]] = "webhook_endpoint"
    api_version: Optional[str]
    """
    The API version events are rendered as for this webhook endpoint.
    """
    application: Optional[str]
    """
    The ID of the associated Connect application.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    deleted: Optional[Literal[True]]
    """
    Always true for a deleted object
    """
    description: Optional[str]
    """
    An optional description of what the webhook is used for.
    """
    enabled_events: List[str]
    """
    The list of events to enable for this endpoint. `['*']` indicates that all events are enabled, except those that require explicit selection.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    metadata: Dict[str, str]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    object: Literal["webhook_endpoint"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    secret: Optional[str]
    """
    The endpoint's secret, used to generate [webhook signatures](https://docs.stripe.com/webhooks/signatures). Only returned at creation.
    """
    status: str
    """
    The status of the webhook. It can be `enabled` or `disabled`.
    """
    url: str
    """
    The URL of the webhook endpoint.
    """

    @classmethod
    def create(
        cls, **params: Unpack["WebhookEndpointCreateParams"]
    ) -> "WebhookEndpoint":
        """
        A webhook endpoint must have a url and a list of enabled_events. You may optionally specify the Boolean connect parameter. If set to true, then a Connect webhook endpoint that notifies the specified url about events from all connected accounts is created; otherwise an account webhook endpoint that notifies the specified url only about events from your account is created. You can also create webhook endpoints in the [webhooks settings](https://dashboard.stripe.com/account/webhooks) section of the Dashboard.
        """
        return cast(
            "WebhookEndpoint",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["WebhookEndpointCreateParams"]
    ) -> "WebhookEndpoint":
        """
        A webhook endpoint must have a url and a list of enabled_events. You may optionally specify the Boolean connect parameter. If set to true, then a Connect webhook endpoint that notifies the specified url about events from all connected accounts is created; otherwise an account webhook endpoint that notifies the specified url only about events from your account is created. You can also create webhook endpoints in the [webhooks settings](https://dashboard.stripe.com/account/webhooks) section of the Dashboard.
        """
        return cast(
            "WebhookEndpoint",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_delete(
        cls, sid: str, **params: Unpack["WebhookEndpointDeleteParams"]
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "WebhookEndpoint",
            cls._static_request(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    def delete(
        sid: str, **params: Unpack["WebhookEndpointDeleteParams"]
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        ...

    @overload
    def delete(
        self, **params: Unpack["WebhookEndpointDeleteParams"]
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        ...

    @class_method_variant("_cls_delete")
    def delete(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["WebhookEndpointDeleteParams"]
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        return self._request_and_refresh(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    async def _cls_delete_async(
        cls, sid: str, **params: Unpack["WebhookEndpointDeleteParams"]
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(sid))
        return cast(
            "WebhookEndpoint",
            await cls._static_request_async(
                "delete",
                url,
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def delete_async(
        sid: str, **params: Unpack["WebhookEndpointDeleteParams"]
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        ...

    @overload
    async def delete_async(
        self, **params: Unpack["WebhookEndpointDeleteParams"]
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        ...

    @class_method_variant("_cls_delete_async")
    async def delete_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["WebhookEndpointDeleteParams"]
    ) -> "WebhookEndpoint":
        """
        You can also delete webhook endpoints via the [webhook endpoint management](https://dashboard.stripe.com/account/webhooks) page of the Stripe dashboard.
        """
        return await self._request_and_refresh_async(
            "delete",
            self.instance_url(),
            params=params,
        )

    @classmethod
    def list(
        cls, **params: Unpack["WebhookEndpointListParams"]
    ) -> ListObject["WebhookEndpoint"]:
        """
        Returns a list of your webhook endpoints.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["WebhookEndpointListParams"]
    ) -> ListObject["WebhookEndpoint"]:
        """
        Returns a list of your webhook endpoints.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["WebhookEndpointModifyParams"]
    ) -> "WebhookEndpoint":
        """
        Updates the webhook endpoint. You may edit the url, the list of enabled_events, and the status of your endpoint.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "WebhookEndpoint",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["WebhookEndpointModifyParams"]
    ) -> "WebhookEndpoint":
        """
        Updates the webhook endpoint. You may edit the url, the list of enabled_events, and the status of your endpoint.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "WebhookEndpoint",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["WebhookEndpointRetrieveParams"]
    ) -> "WebhookEndpoint":
        """
        Retrieves the webhook endpoint with the given ID.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["WebhookEndpointRetrieveParams"]
    ) -> "WebhookEndpoint":
        """
        Retrieves the webhook endpoint with the given ID.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.billing_portal._configuration import Configuration
    from stripe.params.billing_portal._configuration_create_params import (
        ConfigurationCreateParams,
    )
    from stripe.params.billing_portal._configuration_list_params import (
        ConfigurationListParams,
    )
    from stripe.params.billing_portal._configuration_retrieve_params import (
        ConfigurationRetrieveParams,
    )
    from stripe.params.billing_portal._configuration_update_params import (
        ConfigurationUpdateParams,
    )


class ConfigurationService(StripeService):
    def list(
        self,
        params: Optional["ConfigurationListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Configuration]":
        """
        Returns a list of configurations that describe the functionality of the customer portal.
        """
        return cast(
            "ListObject[Configuration]",
            self._request(
                "get",
                "/v1/billing_portal/configurations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["ConfigurationListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[Configuration]":
        """
        Returns a list of configurations that describe the functionality of the customer portal.
        """
        return cast(
            "ListObject[Configuration]",
            await self._request_async(
                "get",
                "/v1/billing_portal/configurations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "ConfigurationCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Configuration":
        """
        Creates a configuration that describes the functionality and behavior of a PortalSession
        """
        return cast(
            "Configuration",
            self._request(
                "post",
                "/v1/billing_portal/configurations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "ConfigurationCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Configuration":
        """
        Creates a configuration that describes the functionality and behavior of a PortalSession
        """
        return cast(
            "Configuration",
            await self._request_async(
                "post",
                "/v1/billing_portal/configurations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        configuration: str,
        params: Optional["ConfigurationRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Configuration":
        """
        Retrieves a configuration that describes the functionality of the customer portal.
        """
        return cast(
            "Configuration",
            self._request(
                "get",
                "/v1/billing_portal/configurations/{configuration}".format(
                    configuration=sanitize_id(configuration),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        configuration: str,
        params: Optional["ConfigurationRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Configuration":
        """
        Retrieves a configuration that describes the functionality of the customer portal.
        """
        return cast(
            "Configuration",
            await self._request_async(
                "get",
                "/v1/billing_portal/configurations/{configuration}".format(
                    configuration=sanitize_id(configuration),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        configuration: str,
        params: Optional["ConfigurationUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Configuration":
        """
        Updates a configuration that describes the functionality of the customer portal.
        """
        return cast(
            "Configuration",
            self._request(
                "post",
                "/v1/billing_portal/configurations/{configuration}".format(
                    configuration=sanitize_id(configuration),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        configuration: str,
        params: Optional["ConfigurationUpdateParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Configuration":
        """
        Updates a configuration that describes the functionality of the customer portal.
        """
        return cast(
            "Configuration",
            await self._request_async(
                "post",
                "/v1/billing_portal/configurations/{configuration}".format(
                    configuration=sanitize_id(configuration),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._customer import Customer
    from stripe.billing._meter import Meter
    from stripe.params.billing._alert_activate_params import (
        AlertActivateParams,
    )
    from stripe.params.billing._alert_archive_params import AlertArchiveParams
    from stripe.params.billing._alert_create_params import AlertCreateParams
    from stripe.params.billing._alert_deactivate_params import (
        AlertDeactivateParams,
    )
    from stripe.params.billing._alert_list_params import AlertListParams
    from stripe.params.billing._alert_retrieve_params import (
        AlertRetrieveParams,
    )


class Alert(CreateableAPIResource["Alert"], ListableAPIResource["Alert"]):
    """
    A billing alert is a resource that notifies you when a certain usage threshold on a meter is crossed. For example, you might create a billing alert to notify you when a certain user made 100 API requests.
    """

    OBJECT_NAME: ClassVar[Literal["billing.alert"]] = "billing.alert"

    class UsageThreshold(StripeObject):
        class Filter(StripeObject):
            customer: Optional[ExpandableField["Customer"]]
            """
            Limit the scope of the alert to this customer ID
            """
            type: Literal["customer"]

        filters: Optional[List[Filter]]
        """
        The filters allow limiting the scope of this usage alert. You can only specify up to one filter at this time.
        """
        gte: int
        """
        The value at which this alert will trigger.
        """
        meter: ExpandableField["Meter"]
        """
        The [Billing Meter](https://docs.stripe.com/api/billing/meter) ID whose usage is monitored.
        """
        recurrence: Literal["one_time"]
        """
        Defines how the alert will behave.
        """
        _inner_class_types = {"filters": Filter}

    alert_type: Literal["usage_threshold"]
    """
    Defines the type of the alert.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    object: Literal["billing.alert"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    status: Optional[Literal["active", "archived", "inactive"]]
    """
    Status of the alert. This can be active, inactive or archived.
    """
    title: str
    """
    Title of the alert.
    """
    usage_threshold: Optional[UsageThreshold]
    """
    Encapsulates configuration of the alert to monitor usage on a specific [Billing Meter](https://stripe.com/docs/api/billing/meter).
    """

    @classmethod
    def _cls_activate(
        cls, id: str, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        return cast(
            "Alert",
            cls._static_request(
                "post",
                "/v1/billing/alerts/{id}/activate".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def activate(id: str, **params: Unpack["AlertActivateParams"]) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        ...

    @overload
    def activate(self, **params: Unpack["AlertActivateParams"]) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        ...

    @class_method_variant("_cls_activate")
    def activate(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        return cast(
            "Alert",
            self._request(
                "post",
                "/v1/billing/alerts/{id}/activate".format(
                    id=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_activate_async(
        cls, id: str, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        return cast(
            "Alert",
            await cls._static_request_async(
                "post",
                "/v1/billing/alerts/{id}/activate".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def activate_async(
        id: str, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        ...

    @overload
    async def activate_async(
        self, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        ...

    @class_method_variant("_cls_activate_async")
    async def activate_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertActivateParams"]
    ) -> "Alert":
        """
        Reactivates this alert, allowing it to trigger again.
        """
        return cast(
            "Alert",
            await self._request_async(
                "post",
                "/v1/billing/alerts/{id}/activate".format(
                    id=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def _cls_archive(
        cls, id: str, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        return cast(
            "Alert",
            cls._static_request(
                "post",
                "/v1/billing/alerts/{id}/archive".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def archive(id: str, **params: Unpack["AlertArchiveParams"]) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        ...

    @overload
    def archive(self, **params: Unpack["AlertArchiveParams"]) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        ...

    @class_method_variant("_cls_archive")
    def archive(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        return cast(
            "Alert",
            self._request(
                "post",
                "/v1/billing/alerts/{id}/archive".format(
                    id=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_archive_async(
        cls, id: str, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        return cast(
            "Alert",
            await cls._static_request_async(
                "post",
                "/v1/billing/alerts/{id}/archive".format(id=sanitize_id(id)),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def archive_async(
        id: str, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        ...

    @overload
    async def archive_async(
        self, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        ...

    @class_method_variant("_cls_archive_async")
    async def archive_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertArchiveParams"]
    ) -> "Alert":
        """
        Archives this alert, removing it from the list view and APIs. This is non-reversible.
        """
        return cast(
            "Alert",
            await self._request_async(
                "post",
                "/v1/billing/alerts/{id}/archive".format(
                    id=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def create(cls, **params: Unpack["AlertCreateParams"]) -> "Alert":
        """
        Creates a billing alert
        """
        return cast(
            "Alert",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["AlertCreateParams"]
    ) -> "Alert":
        """
        Creates a billing alert
        """
        return cast(
            "Alert",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def _cls_deactivate(
        cls, id: str, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        return cast(
            "Alert",
            cls._static_request(
                "post",
                "/v1/billing/alerts/{id}/deactivate".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def deactivate(
        id: str, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        ...

    @overload
    def deactivate(self, **params: Unpack["AlertDeactivateParams"]) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        ...

    @class_method_variant("_cls_deactivate")
    def deactivate(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        return cast(
            "Alert",
            self._request(
                "post",
                "/v1/billing/alerts/{id}/deactivate".format(
                    id=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_deactivate_async(
        cls, id: str, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        return cast(
            "Alert",
            await cls._static_request_async(
                "post",
                "/v1/billing/alerts/{id}/deactivate".format(
                    id=sanitize_id(id)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def deactivate_async(
        id: str, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        ...

    @overload
    async def deactivate_async(
        self, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        ...

    @class_method_variant("_cls_deactivate_async")
    async def deactivate_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["AlertDeactivateParams"]
    ) -> "Alert":
        """
        Deactivates this alert, preventing it from triggering.
        """
        return cast(
            "Alert",
            await self._request_async(
                "post",
                "/v1/billing/alerts/{id}/deactivate".format(
                    id=sanitize_id(self.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def list(cls, **params: Unpack["AlertListParams"]) -> ListObject["Alert"]:
        """
        Lists billing active and inactive alerts
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["AlertListParams"]
    ) -> ListObject["Alert"]:
        """
        Lists billing active and inactive alerts
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["AlertRetrieveParams"]
    ) -> "Alert":
        """
        Retrieves a billing alert given an ID
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["AlertRetrieveParams"]
    ) -> "Alert":
        """
        Retrieves a billing alert given an ID
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"usage_threshold": UsageThreshold}
